#include <metal_stdlib>
#include <CoreImage/CoreImage.h>

// modified from OpenGL Source: https://gist.github.com/mattatz/44f081cac87e2f7c8980
// http://www.easyrgb.com/en/math.php#text8

using namespace metal;

float3 rgb2xyz( float3 c ) {
    float3 tmp;
    tmp.x = (c.r > 0.04045) ? pow((c.r + 0.055) / 1.055, 2.4) : c.r / 12.92;
    tmp.y = (c.g > 0.04045) ? pow((c.g + 0.055) / 1.055, 2.4) : c.g / 12.92,
    tmp.z = (c.b > 0.04045) ? pow((c.b + 0.055) / 1.055, 2.4) : c.b / 12.92;
    const float3x3 mat = float3x3(
        0.41239080, 0.35758434, 0.18048079,
        0.21263901, 0.71516868, 0.07219232,
        0.01933082, 0.11919478, 0.95053215
    );
    return 100.0 * (tmp * mat);
}

float3 xyz2lab( float3 c ) {
    float3 n = c / float3(95.047, 100, 108.883);
    float3 v;
    v.x = (n.x > 0.008856) ? pow(n.x, 1.0 / 3.0) : (7.787 * n.x) + (16.0 / 116.0);
    v.y = (n.y > 0.008856) ? pow(n.y, 1.0 / 3.0) : (7.787 * n.y) + (16.0 / 116.0);
    v.z = (n.z > 0.008856) ? pow(n.z, 1.0 / 3.0) : (7.787 * n.z) + (16.0 / 116.0);
    return float3((116.0 * v.y) - 16.0, 500.0 * (v.x - v.y), 200.0 * (v.y - v.z));
}

float3 rgb2lab( float3 c ) {
    float3 lab = xyz2lab(rgb2xyz(c));
    return float3(lab.x / 100.0, 0.5 + 0.5 * (lab.y / 127.0), 0.5 + 0.5 * (lab.z / 127.0));
}

float3 lab2xyz( float3 c ) {
    float fy = ( c.x + 16.0 ) / 116.0;
    float fx = c.y / 500.0 + fy;
    float fz = fy - c.z / 200.0;
    return float3(
         95.047 * ((fx > 0.206897) ? fx * fx * fx : (fx - 16.0 / 116.0) / 7.787),
        100.000 * ((fy > 0.206897) ? fy * fy * fy : (fy - 16.0 / 116.0) / 7.787),
        108.883 * ((fz > 0.206897) ? fz * fz * fz : (fz - 16.0 / 116.0) / 7.787)
    );
}

float3 xyz2rgb( float3 c ) {
    const float3x3 mat = float3x3(
         3.24096944, -1.53738318, -0.49861076,
        -0.96924364,  1.8759675,   0.04155506,
         0.05563008, -0.20397696,  1.05697151
    );
    float3 v = (c / 100.0) * mat;
    float3 r;
    r.x = (v.r > 0.0031308) ? ((1.055 * pow(v.r, ( 1.0 / 2.4))) - 0.055) : 12.92 * v.r;
    r.y = (v.g > 0.0031308) ? ((1.055 * pow(v.g, ( 1.0 / 2.4))) - 0.055) : 12.92 * v.g;
    r.z = (v.b > 0.0031308) ? ((1.055 * pow(v.b, ( 1.0 / 2.4))) - 0.055) : 12.92 * v.b;
    return r;
}

float3 lab2rgb(float3 c) {
    return xyz2rgb(lab2xyz(float3(100.0 * c.x, 2.0 * 127.0 * (c.y - 0.5), 2.0 * 127.0 * (c.z - 0.5))));
}


extern "C" float4 LABInvert (coreimage::sample_t s, coreimage::destination dest) {
    float3 lab = rgb2lab(float3(s));
    lab[0] = 1.0 - lab[0];
    if (lab[0] < 0.025) {
        lab[0] = 0.025;
    }
    if (lab[0] > 0.9) {
        lab[0] = 0.9;
    }
    float3 rgb = lab2rgb(lab);
    return float4(rgb, s.a);
}
