// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Combine
import UIKit
import TUMexamKit
import Logging

extension CorrectionCanvasViewController: CorrectionViewSnippetCreationElementDelegate {
	func snippetCreation(_ element: CorrectionViewSnippetCreationElement, didSelectPath normalizedPath: CGPath) {
		// get necessary kit objects
		guard let pass = page.focusedPass,
			  let layer = page.layer(pass: pass),
			  let drawing = layer.drawing.value else {
			// maybe show error?
			return
		}

		// get necessary UI objects
		guard let exam = App.shared.currentExam, let target = canvasContext.snippetTarget else { return }

		// try getting image
		guard let cgImage = drawing.getImage() else { return }
		let image = UIImage(cgImage: cgImage)

		// try cutout
		guard let cutout = image.cutout(normalizedPath: normalizedPath) else {
			return
		}
		guard let (cropped, hasAlpha) = cutout.cropUsingAlphaChannel(extend: 5) else {
			return
		}
		guard hasAlpha else {
            Logger.correctionUI.error("[CanvasVC|\(page.erid ?? "nil")|\(page.pageNumber)] Snippet has no content")
			return
		}

		let snippets = exam.snippets
		let snippet = snippets.addSnippet(problem: target, name: "", image: cropped)

		if let boxGroup = boxes?.boxGroup(forProblem: target), let source = boxGroup.actionButton {
			showSnippets(forProblem: boxGroup.problem, source: source, focus: snippet)
		}

		canvasContext.setMode(.normal)
	}
}

extension CorrectionCanvasViewController: SnippetsTableViewControllerDelegate {
	func snippetsViewController(_ viewController: SnippetsTableViewController, didSelect snippet: SnippetType) {
		viewController.navigationController?.dismiss(animated: true, completion: nil)

		canvasContext.startSnippetPlacement(snippet: snippet)
	}

	func snippetsViewControllerWantsToAdd(_ viewController: SnippetsTableViewController) {
		canvasContext.startSnippetCreation(target: viewController.problem.identifier)
		viewController.navigationController?.dismiss(animated: true, completion: nil)
	}
}

extension CorrectionCanvasViewController: CorrectionViewSnippetPlacementElementDelegate {
	func snippetPlacement(_ element: CorrectionViewSnippetPlacementElement,
						  didSelectRect rect: CGRect,
						  forSnippet snippet: SnippetType) {
		let element = SnippetElement(snippet: snippet, normalizedRect: rect)
		self.drawing?.canvasView?.drawing.append(element: .snippet(element))
		canvasContext.setMode(.normal)
	}
}
