// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit
import os.log

class CorrectionViewEmptyBoxElement: CorrectionViewElement {
    var emptyBox: EmptyBoxType?
    var button: CorrectionBoxButton?
    var overlayMode: Bool = false {
        didSet {
            button?.overlayMode = overlayMode
        }
    }

    override func setup() {
        super.setup()

        guard let emptyBox = context.page.emptyBox.value else {
            return
        }
        self.emptyBox = emptyBox

        // create button using a default rect
        let emptyFrame = CGRect(origin: .zero, size: CGSize(width: 10.0, height: 10.0))
        let button = CorrectionBoxButton(frame: emptyFrame, box: emptyBox)

        button.normalizedFrame = emptyBox.normalizedFrame.normalizedIn(rect: context.normalizedPageRect)

        // Add CorrectionBoxButton to view and keep a reference
        self.addSubview(button)
        self.button = button
    }

    override func layoutSubviews() {
        super.layoutSubviews()

        let boxFrame = context.page.emptyBox.value?.normalizedFrame
            .normalizedIn(rect: context.normalizedPageRect)
            .mappedTo(size: self.bounds.size)
        button?.frame = boxFrame ?? .zero
        button?.isHidden = !(button?.frame.intersects(bounds) ?? false)
    }

    override func point(inside point: CGPoint, with event: UIEvent?) -> Bool {
        return button?.frame.contains(point) ?? false
    }
}
