// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Combine
import UIKit
import TUMexamKit

class CorrectionCanvasContext {
	let page: PageType
	@Published var normalizedPageRect: CGRect
	@Published var undoManager: ContextUndoManager?

	var color: UIColor

    enum Mode {
        case normal
        case snippetCreation
        case snippetPlacement
		case selectionInsertion
		case selectionPlacement
    }

    @Published private(set) var mode: Mode = .normal

	// MARK: - Snippets

	var snippetTarget: ProblemIdentifier?
    var snippetToPlace: SnippetType?
    var snippetInitialLocation: CGPoint?

	// MARK: - Init

	init(page: PageType, undoManager: ContextUndoManager = ContextUndoManager()) {
		self.page = page
		self.normalizedPageRect = CGRect(x: 0, y: 0, width: 1, height: 1)

		switch page.focusedPass {
		case 2:
			self.color = Asset.correctionPass2.color
		case 3:
			self.color = Asset.correctionPass3.color
		default:
			self.color = Asset.correctionPass1.color
		}
	}

    func setMode(_ mode: Mode) {
        guard mode != self.mode else { return }
        self.mode = mode
    }

    func startSnippetPlacement(snippet: SnippetType, initialLocation: CGPoint? = nil) {
        snippetToPlace = snippet
        snippetInitialLocation = initialLocation
        snippetTarget = nil
        setMode(.snippetPlacement)
    }

    func startSnippetCreation(target: ProblemIdentifier) {
        snippetTarget = target
        snippetToPlace = nil
        setMode(.snippetCreation)
    }

}
