// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import Combine

class CorrectionViewElement: BaseView {
	let context: CorrectionCanvasContext
	var normalizedPageRectSubscriber: AnyCancellable?

	init(context: CorrectionCanvasContext) {
		self.context = context
		super.init()
	}

	required init?(coder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}

	override func setup() {
		super.setup()
        self.layer.masksToBounds = true

		normalizedPageRectSubscriber = context.$normalizedPageRect.receiveOnMain().sink(receiveValue: { [weak self] rect in
			self?.normalizedPageRectDidChange(rect)
		})
	}

	func normalizedPageRectDidChange(_ rect: CGRect) {
	}
}
