// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Combine
import UIKit
import TUMexamKit
import os.log

class CorrectionViewDrawingElement: CorrectionViewElement {
    var tool: Tool = .pen {
        didSet {
            canvasView?.tool = tool
        }
    }
	var canvasView: CanvasView?
	var targetLayer: LayerType?

	var croppedImage: CroppedImage?

	var imageObserver: AnyCancellable?
	var scanObserver: AnyCancellable?
	var undoManagerObserver: AnyCancellable?
	var drawingObserver: AnyCancellable?

	override func setup() {
		super.setup()

		if let pass = context.page.focusedPass,
		   let layer = context.page.layer(pass: pass) {
			self.targetLayer = layer
            drawingObserver = layer.drawing.receiveOnMain().sink { [weak self] (drawing) in
				guard let drawing = drawing else { return }
				self?.drawingAvailable(drawing)
			}
		} else {
			fatalError("couldn't get layer!")
		}
	}

	override func layoutSubviews() {
		super.layoutSubviews()

		canvasView?.frame = self.bounds
	}

	func drawingAvailable(_ drawing: Drawing) {
		var adjustedDrawing: Drawing = drawing
		if context.normalizedPageRect.size != CGSize(width: 1.0, height: 1.0) {
			adjustedDrawing = drawing.createSubdrawing(normalizedRect: context.normalizedPageRect)
		}

        if let oldCanvas = canvasView {
            oldCanvas.removeFromSuperview()
        }

		let canvas = CanvasView(drawing: adjustedDrawing)
		canvas.color = context.color
        canvas.tool = tool
		canvasView = canvas
		addSubview(canvas)

		undoManagerObserver = context.$undoManager.receiveOnMain().assignNoRetain(to: \.contextUndoManager, on: canvas)
	}
}
