// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class CorrectionViewEmptyElement: CorrectionViewElement {
	let label: UILabel = UILabel().autolayout()
    var activatedByCrossbox: Bool = false {
        didSet {
            if activatedByCrossbox {
                label.text = L10n.CorrectionController.Canvas.emptyByCrossbox
            } else {
                var text: String = L10n.CorrectionController.Canvas.empty
                if let problem = context.page.focusedProblem, let pass = context.page.focusedPass {
                    text += " – " + L10n.CorrectionController.ProblemPass.title("\(problem)", "\(pass)")
                }
                label.text = text
            }
        }
    }

	override func setup() {
		super.setup()

		var text: String = L10n.CorrectionController.Canvas.empty
		if let problem = context.page.focusedProblem, let pass = context.page.focusedPass {
			text += " – " + L10n.CorrectionController.ProblemPass.title("\(problem)", "\(pass)")
		}
		label.text = text
		label.textColor = .red
		label.textAlignment = .center
		label.font = UIFont.boldSystemFont(ofSize: 50)

		addSubview(label)
	}

	override func layoutSubviews() {
		super.layoutSubviews()

		label.transform = .identity
		label.frame = CGRect(center: CGPoint(x: self.bounds.size.width / 2,
											 y: self.bounds.size.height / 2),
                             size: CGSize(width: self.bounds.size.width * 1.5,
										  height: 60))
        if 0.5 * sin(45) * label.frame.size.width > self.bounds.size.height/2 {
            label.transform = CGAffineTransform.identity
        } else {
            label.transform = label.transform.rotated(by: 45.0 * .pi / 180)
        }
	}
}
