// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import Combine
import TUMexamKit
import Logging

class CorrectionViewImageElement: CorrectionViewElement {
	var image: CroppedImage? {
		didSet {
            self.updateImageView()
		}
	}

	var imageView: UIImageView = UIImageView().autolayout()

	open var loadableImage: LoadableImageType { fatalError("implement in subclass") }

	private var imageSubscriber: AnyCancellable?

	override func setup() {
		super.setup()

		self.addSubview(imageView)
		imageView.pinToSuperview()

        imageSubscriber = loadableImage.image
            .receiveOnMain()
            .map({ [weak self] (image) -> CroppedImage? in
                guard let cgImage = image?.cgImage, let pageRect = self?.context.normalizedPageRect else { return nil }
                return CroppedImage(cgImage, croppedToNormalized: pageRect)
            })
            .assignNoRetain(to: \.image, on: self)
	}

	override func normalizedPageRectDidChange(_ rect: CGRect) {
		super.normalizedPageRectDidChange(rect)

		image?.crop(toNormalized: rect)
		updateImageView()
	}

	func updateImageView() {
		guard let image = image else {
			imageView.image = nil
			return
		}

		imageView.image = UIImage(cgImage: image.cropped)
	}
}
