// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

//class CorrectionViewScanElement: CorrectionViewTilingImageElement {
//
//    override var loadableImage: LoadableImageType {
//        context.page.scan
//    }
//
//}

class CorrectionViewScanElement: CorrectionViewImageElement {
	override var loadableImage: LoadableImageType {
		context.page.scan
	}

    private var darkModeImage: CGImage?

    override func updateImageView() {
        guard let image = image?.cropped else {
            imageView.image = nil
            return
        }

        darkMode: if App.shared.settings[SettingsKeys.enhancedDarkMode] as? Bool ?? false {
            // Apply Filter
            let ctx = CIContext()
            let ciImage = CIImage(cgImage: image)
            let filter = LABInvertFilter()
            filter.inputImage = ciImage
            guard let darkCIImage = filter.outputImage else { break darkMode }
            let darkCG = ctx.createCGImage(darkCIImage,
                                              from: darkCIImage.extent,
                                              format: .RGBA8,
                                              colorSpace: CGColorSpaceCreateDeviceRGB())
            guard let darkImage = darkCG else { break darkMode }

            self.darkModeImage = darkImage

            // Set image if darkmode is on
            if traitCollection.userInterfaceStyle == .dark {
                imageView.image = UIImage(cgImage: darkImage)
                return
            }
        }

        imageView.image = UIImage(cgImage: image)
    }

    override func traitCollectionDidChange(_ previousTraitCollection: UITraitCollection?) {
        super.traitCollectionDidChange(previousTraitCollection)

        guard App.shared.settings[SettingsKeys.enhancedDarkMode] as? Bool ?? false else {
            return
        }

        switch traitCollection.userInterfaceStyle {
        case .dark:
            guard let darkModeImage = darkModeImage else {
                return
            }
            imageView.image = UIImage(cgImage: darkModeImage)
        default:
            guard let image = image?.cropped else {
                imageView.image = nil
                return
            }
            imageView.image = UIImage(cgImage: image)
        }
    }
}
