// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class CorrectionViewLockStateElement: CorrectionViewElement {
    let button: UIButton = UIButton(type: .custom).autolayout()

    var retryHandler: (() -> Void)?

    override func setup() {
        super.setup()

        button.setTitle(L10n.CorrectionController.Canvas.noLock, for: .normal)
        button.setTitleColor(.black, for: .normal)
        button.titleLabel?.textAlignment = .center
        button.titleLabel?.font = UIFont.preferredFont(forTextStyle: .headline)

        button.backgroundColor = UIColor.systemRed.withAlphaComponent(0.75)

        button.addTarget(self, action: #selector(didTapButton), for: .touchUpInside)

//        container.addSubview(label)
        addSubview(button)

//        label.pinToSuperview(UIEdgeInsets(top: 4, left: 0, bottom: 4, right: 0))

        NSLayoutConstraint.activate([
            button.topAnchor.constraint(equalTo: topAnchor),
            button.leadingAnchor.constraint(equalTo: leadingAnchor),
            button.trailingAnchor.constraint(equalTo: trailingAnchor)
        ])
    }

    @objc
    func didTapButton() {
        retryHandler?()
    }
}
