// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import TUMexamKit
import UIKit

protocol LassoSelectionDelegate: AnyObject {
	func showSelectionInsertion(_ element: CorrectionViewSelectionInsertionElement,
								at point: CGPoint)
	func showSelectionPlacement(_ element: CorrectionViewSelectionPlacementElement,
								didSelectRect rect: CGRect,
								forSelection selection: Image)
}

final class CorrectionViewSelectionInsertionElement: CorrectionViewElement {

	var selection: Image? {
		didSet {
			if selection == nil {
				removeGestureRecognizer(tapGestureRecognizer)
			} else {
				addGestureRecognizer(tapGestureRecognizer)
			}
		}
	}

	weak var delegate: LassoSelectionDelegate?

	lazy var tapGestureRecognizer: UITapGestureRecognizer = {
		return UITapGestureRecognizer(target: self, action: #selector(didTap(_:)))
	}()

	@objc
	private func didTap(_ recognizer: UITapGestureRecognizer) {
		let point = recognizer.location(in: self)
		delegate?.showSelectionInsertion(self, at: point)
	}

	override var canBecomeFirstResponder: Bool { true }
}
