// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

class CorrectorListTableViewController: UITableViewController {

    let dateFormatter: DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateStyle = .medium
        formatter.timeStyle = .short
        return formatter
    }()
    var correctorData = [SubproblemCorrectorType]()
    private var contentSizeObserver: NSKeyValueObservation?

    override func viewDidLoad() {
        super.viewDidLoad()

        self.title = L10n.CorrectionController.PreviousCorrectors.title
        self.tableView.allowsSelection = false
        self.tableView.register(UITableViewCell.self, forCellReuseIdentifier: "correctorList")

        contentSizeObserver = self.tableView.observe(\.contentSize) { [weak self] tableView, _ in
            self?.preferredContentSize = CGSize(
                width: self?.view.systemLayoutSizeFitting(UIView.layoutFittingCompressedSize).width ?? 300,
                height: tableView.contentSize.height
            )
        }
    }

    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        contentSizeObserver?.invalidate()
        contentSizeObserver = nil
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return correctorData.count
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let sectionData = correctorData[section]
        if sectionData.correctors.compactMap({ return $0 }).isEmpty {
            return 1
        }
        return sectionData.correctors.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "correctorList", for: indexPath)
        let sectionData = correctorData[indexPath.section]

        if sectionData.correctors.compactMap({ return $0 }).isEmpty {
            cell.textLabel?.text = L10n.CorrectionController.PreviousCorrectors.noDataAvailable
            cell.textLabel?.textColor = .secondaryLabel
            cell.textLabel?.font = .italicSystemFont(ofSize: cell.textLabel!.font.pointSize)
            return cell
        }

        guard let rowData = sectionData.correctors[indexPath.row] else {
            cell.textLabel?.text = L10n.CorrectionController.PreviousCorrectors.cellContent(indexPath.row + 1, "N/A")
            return cell
        }

        cell.detailTextLabel?.text =  dateFormatter.string(from: rowData.date)
        cell.textLabel?.text = L10n.CorrectionController.PreviousCorrectors.cellContent(indexPath.row + 1, rowData.name)

        return cell
    }

    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        let sectionData = correctorData[section]
        return sectionData.subproblem
    }
}
