// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit
import Instructions

extension EditorWrapperViewController {
    func setupCoachMarks() {
        coachContent = [
            (L10n.Tutorial.PagingCorrection.pageCon, self.editorContent.view, self.view.center, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.tools, self.toolbarManager?.penBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.undo,
             self.toolbarManager?.undoBarItem?.value(forKey: "view") as? UIView, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.empty, self.toolbarManager?.emptyBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.solution, self.toolbarManager?.solutionBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.personalizedSolution,
             self.toolbarManager?.personalizedSolutionBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.raw, self.toolbarManager?.rawBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.overlay, self.toolbarManager?.overlayBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.subpropblem,
             self.toolbarManager?.subproblemSelectionBarItem?.button, nil, nil),
            (L10n.Tutorial.PagingCorrection.toolbarSettings,
             self.toolbarManager?.toolbarSettingsItem?.value(forKey: "view") as? UIView, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.save,
             self.toolbarManager?.saveBarItem?.value(forKey: "view") as? UIView, nil, nil),
            (L10n.Tutorial.PagingCorrection.Navbar.dismiss,
             self.toolbarManager?.dismissBarItem?.value(forKey: "view") as? UIView, nil, nil)
        ]
    }
}

extension EditorWrapperViewController: CoachMarksControllerDelegate {
    func coachMarksController(_ coachMarksController: CoachMarksController, didEndShowingBySkipping skipped: Bool) {
        App.shared.settings[HiddenSettings.didShowPagingCorrectionTutorial] = true
    }
}

extension EditorWrapperViewController: CoachMarksControllerDataSource {
    func numberOfCoachMarks(for coachMarksController: CoachMarksController) -> Int {
        return coachContent.count
    }

    func coachMarksController(_ coachMarksController: CoachMarksController, coachMarkAt index: Int) -> CoachMark {
        let content = coachContent[index]
        var mark = coachMarksController.helper.makeCoachMark(for: content.1,
                                                             pointOfInterest: content.2,
                                                             cutoutPathMaker: content.3)
        if index == 0 {
            mark.isDisplayedOverCutoutPath = true
        }

        return mark
    }

    func coachMarksController(_ coachMarksController: CoachMarksController,
                              coachMarkViewsAt index: Int,
                              madeFrom coachMark: CoachMark) -> (bodyView: (UIView & CoachMarkBodyView),
                                                                 arrowView: (UIView & CoachMarkArrowView)?) {
        let coachViews = coachMarksController.helper.makeDefaultCoachViews(withArrow: true,
                                                                           arrowOrientation: coachMark.arrowOrientation)
        coachViews.bodyView.hintLabel.text = coachContent[index].0
        coachViews.bodyView.nextLabel.text = L10n.gotIt

        return (coachViews.bodyView, coachViews.arrowView)
    }

    func coachMarksController(_ coachMarksController: CoachMarksController,
                              constraintsForSkipView skipView: UIView,
                              inParent parentView: UIView) -> [NSLayoutConstraint]? {
        return [
            skipView.bottomAnchor.constraint(equalTo: parentView.safeAreaLayoutGuide.bottomAnchor),
            skipView.centerXAnchor.constraint(equalTo: parentView.centerXAnchor)
        ]
    }
}
