// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit
import UIKitCore
import SAConfettiView

class EndViewController: UIViewController {

    private let pass: Int

    private let confettiView = SAConfettiView().autolayout()
    private let titleLabel = UILabel().autolayout()
    private let subtitleLabel = UILabel().autolayout()

    init(_ pass: Int = 1) {
        self.pass = pass
        super.init(nibName: nil, bundle: nil)
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        self.view.addSubview(confettiView)
        confettiView.pinToEdges()
        confettiView.type = .confetti
        confettiView.intensity = 0.75

        self.view.addSubview(titleLabel)
        titleLabel.text = pass == 1 ?
                            L10n.CorrectionController.End.title : L10n.CorrectionController.End.Second.title
        titleLabel.font = .boldSystemFont(ofSize: 72)
        titleLabel.textColor = .label
        titleLabel.numberOfLines = 0
        titleLabel.minimumScaleFactor = 0.3
        titleLabel.centerX(to: \.centerXAnchor)
        titleLabel.centerY(to: \.centerYAnchor, constant: -20)

        self.view.addSubview(subtitleLabel)
        subtitleLabel.text = pass == 1 ?
                                L10n.CorrectionController.End.subtitle : L10n.CorrectionController.End.Second.subtitle
        subtitleLabel.textAlignment = .center
        subtitleLabel.numberOfLines = 2
        subtitleLabel.textColor = .secondaryLabel
        subtitleLabel.centerX(to: \.centerXAnchor)
        subtitleLabel.top(to: \.bottomAnchor, of: titleLabel, constant: 20)
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        confettiView.isHidden = false
        confettiView.startConfetti()
    }

    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        confettiView.stopConfetti()
        UIView.animate(withDuration: 0.2) {
            self.confettiView.isHidden = true
        }
    }

}
