// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class LoadingViewController: UIViewController {
	enum Style {
		case page
		case clear
	}

	var style: Style = .clear {
		didSet {
			if oldValue != style {
				updateStyle()
			}
		}
	}

	let activityIndicator: UIActivityIndicatorView = UIActivityIndicatorView(style: .large).autolayout()
	let label: UILabel = UILabel().autolayout()
	let stack = UIStackView().autolayout()

	init() {
		super.init(nibName: nil, bundle: nil)
	}

	init(style: Style) {
		self.style = style
		super.init(nibName: nil, bundle: nil)
	}

	required init?(coder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}

    override func viewDidLoad() {
        super.viewDidLoad()

		label.text = L10n.LoadingController.title

        // Do any additional setup after loading the view.
		stack.axis = .vertical
		stack.alignment = .center
		stack.spacing = 10.0
		stack.addArrangedSubview(activityIndicator)
		stack.addArrangedSubview(label)
		view.addSubview(stack)

		stack.centerInSuperview()

		updateStyle()
    }

	func updateStyle() {
		guard isViewLoaded else { return }
		switch style {
		case .page:
			view.backgroundColor = .white
			activityIndicator.color = .black
			label.textColor = .black
		case .clear:
			view.backgroundColor = .clear
//			activityIndicator.tintColor = nil
		}
	}

	override func viewWillAppear(_ animated: Bool) {
		super.viewWillAppear(animated)
		activityIndicator.startAnimating()
	}

	override func viewWillDisappear(_ animated: Bool) {
		super.viewWillDisappear(animated)
		activityIndicator.stopAnimating()
	}
}
