// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Combine
import TUMexamKit

// wtf this name
class StatefulLoadableSubloadingResource: SubloadingResource {
    let statefulLoadable: StatefulLoadable

    private var stateObserver: AnyCancellable?

    init(_ statefulLoadable: StatefulLoadable, title: String, isMandatory: Bool = true) {
        self.statefulLoadable = statefulLoadable
        super.init(title: title, isMandatory: isMandatory)

        stateObserver = statefulLoadable.state.sink { [weak self] (state) in
            self?.stateChanged(state)
        }
    }

    override func load() {
        // this will directly execute the block with the current state
//        stateObserver = statefulLoadable.state.sink { [weak self] (state) in
//            self?.stateChanged(state)
//        }
        statefulLoadable.load()
    }

    func stateChanged(_ state: ResourceObjectState) {
        switch state {
        case .idle:
            self.loadingItem.state = .waiting
            guard state != .loading else {
                return
            }
            statefulLoadable.load()
        case .loading:
            loading()
        case .loaded:
            done()
        case let .error(error):
            self.error(error)
        }
    }
}
