// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class SubloadingResource {
    let loadingItem: LoadingItem
    /// Some resources, like layer images or solution images, might not exist, we don't want to throw errors because of this
    let isMandatory: Bool

    var onCompletion: ((SubloadingResource) -> Void)?
    var onError: ((SubloadingResource, Error) -> Void)?

    init(title: String, isMandatory: Bool = true) {
        self.loadingItem = LoadingItem(title: title)
        self.isMandatory = isMandatory
    }

    func load() {
        fatalError("override in subclass")
    }

    // MARK: - Convenience Methods for Subclasses

    func loading() {
        loadingItem.state = .loading
    }

    func done() {
        loadingItem.state = .loaded
        onCompletion?(self)
    }

    func error(_ error: Error) {
        // if not mandatory -> no error
        if !self.isMandatory {
            done()
            return
        }
        // mandatory -> error
        loadingItem.state = .error
        onError?(self, error)
    }
}
