// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

/// base class for something that loads a resource of type T
class LoadingResource<T>: CustomStringConvertible {
    enum State {
        case idle
        case loading
        case content(T)
        case error(Error)
        case reset
    }

    @Published var state: State = .idle

    func load() {

    }

    var description: String { "LoadingResource" }
}

extension LoadingResource.State: Equatable {
    static func == (lhs: LoadingResource<T>.State, rhs: LoadingResource<T>.State) -> Bool {
        switch (lhs, rhs) {
        case (.idle, .idle), (.loading, .loading), (.content, .content), (.error, .error), (.reset, .reset):
            return true
        default:
            return false
        }
    }
}

/// objects conforming to this type provide additional information about their loading state
protocol LoadingItemProvider: AnyObject {
    var loadingTitle: String { get }
    var loadingItems: [LoadingItem] { get }

    var onChange: (() -> Void)? { get set }
}
