// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit
import TUMexamKit

extension CorrectionPageViewController: UIPageViewControllerDataSource {

    func index(of controller: CorrectionViewController) -> Int {
        return controller.page.pageNumber - 1
    }

    func viewController(at index: Int) -> UIViewController? {
        guard index >= 0 && index < self.correction.pages.value.count else { return nil}
        guard let page = self.correction.pages.value[safeAccess: index] else { return nil }
        return cache.viewController(for: page)
    }

    func pageViewController(_ pageViewController: UIPageViewController,
                            viewControllerBefore viewController: UIViewController) -> UIViewController? {
        guard let page = viewController as? CorrectionViewController else { return nil }
        return self.viewController(at: index(of: page) - 1)
    }

    func pageViewController(_ pageViewController: UIPageViewController,
                            viewControllerAfter viewController: UIViewController) -> UIViewController? {
        guard let page = viewController as? CorrectionViewController else { return nil }
        return self.viewController(at: index(of: page) + 1)
    }
}
