// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

extension CorrectionPageViewController {
	func lockStateChanged(_ state: LockState) {
		switch state {
        case .unknown, .freed:
			break
		case .acquired:
			removeTakenLockIndication()
		case .taken:
			indicateTakenLock()
		}
	}

	func restoreLockToast() {
		if let toast = self.lockedToast, !toast.isBeingPresented {
			self.toastPresentingParent()?.show(toast)
		}
	}

	func hideLockToast() {
		if let toast = self.lockedToast, toast.isBeingPresented {
			toast.dismiss()
		}
	}

	func indicateTakenLock() {
        if lockStartedLoad {
            lockStartedLoad = false
            return
        }
        self.lockLoadingToast?.dismiss()
        self.lockLoadingToast = nil

		if self.lockedToast == nil {
			let toast = Toast(
				.error,
				title: L10n.Toast.LockFailed.title,
				subtitle: L10n.Toast.LockFailed.subtitle(self.correction.erid),
				accessory: .symbol("lock.slash"),
				action: .init(title: L10n.Toast.LockFailed.action, handler: { [weak self] (_) in
					self?.requestLock()
				}),
				dismissable: false)
			self.lockedToast = toast
		}
		if self.isCurrentlyAppearing {
			restoreLockToast()
		}
	}

	func removeTakenLockIndication() {
        self.lockedToast?.dismiss()
        self.lockedToast = nil
        self.lockLoadingToast?.dismiss()
        self.lockLoadingToast = nil
	}

	@objc func requestLock() {
        self.lockReference = nil

        self.lockedToast?.dismiss()
        self.lockedToast = nil

        lockStartedLoad = true
        let lockReference = correction.addLockRequirement()

        let loadingToast = Toast(
            .error,
            title: L10n.Loading.Correction.lock,
            accessory: .activityIndicator,
            dismissable: false
        )
        self.lockLoadingToast = loadingToast
        self.toastPresentingParent()?.show(loadingToast)

        lockStateObserver = lockReference.state.receiveOnMain().sink { [weak self] state in
            self?.lockStateChanged(state)
        }
        self.lockReference = lockReference
	}
}
