// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import Logging

extension OuterCorrectionPageViewController: UIPageViewControllerDelegate {
	func pageViewController(
		_ pageViewController: UIPageViewController,
		didFinishAnimating finished: Bool,
		previousViewControllers: [UIViewController],
		transitionCompleted completed: Bool) {
        parent(conformingTo: CorrectionTitleTarget.self)?.updateTitle()
        guard finished else { return }

        // check the previous view controller for changes, and save if necessary
        if let old = previousViewControllers.first as? CorrectionRepresentingViewController,
           let correction = old.representingCorrection {
            if let first = self.viewControllers?.first as? CorrectionRepresentingViewController,
               let currentId = first.representingCorrection?.identifier.stringIdentifier {
                if first === old {
                    // Same VC, do nothing
                    return
                }
                BreadcrumbManager.addNavigation("[OuterCorrectionPageVC] transitioning from \(correction.identifier.stringIdentifier) to \(currentId)")
            }
            if correction.hasChanges {
                Logger.correctionUI.info("[OuterCorrectionPageVC] saving changes because of transition")
                save(correction: correction)
            } else {
                notifyNothingSaved(correction: correction)
            }
        }

        // inform delegate about change
        if let first = self.viewControllers?.first {
        	correctionDataSource.viewControllerDidChange(to: first)
        }

        // Notify view controller so it can update the undo manager (viewDidAppear is not reliable)
        if let wrapper = self.viewControllers?.first as? CorrectionLoadingWrapperViewController,
           let pageController = wrapper.contentViewController {
            pageController.becomeUndoMager()
        } else if let pageController = self.viewControllers?.first as? CorrectionPageViewController {
            pageController.becomeUndoMager()
        }
	}
}
