// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import MBProgressHUD

extension SingleCorrectionViewController {
    func setupPencilInteraction() {
        guard App.shared.settings.applePencilDoubleTapGestureEnabled else { return }
        let pencilInteraction = UIPencilInteraction()
        pencilInteraction.delegate = self
        self.view.addInteraction(pencilInteraction)
        self.pencilInteraction = pencilInteraction
    }
}

extension SingleCorrectionViewController: UIPencilInteractionDelegate {
    func pencilInteractionDidTap(_ interaction: UIPencilInteraction) {
        BreadcrumbManager.addInteraction("[SingleCorrectionVC] pencil interaction")
        guard UIPencilInteraction.preferredTapAction != .ignore else {
            return
        }

        let current = context.drawingTool
        if current == .eraser {
            context.drawingTool = .pen
            let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
            hud.mode = .text
            hud.isUserInteractionEnabled = false
            hud.label.text = L10n.Hud.ToolChange.pen
            hud.hide(animated: true, afterDelay: 0.3)
        } else {
            context.drawingTool = .eraser
            let hud = MBProgressHUD.showAdded(to: self.view, animated: true)
            hud.mode = .text
            hud.isUserInteractionEnabled = false
            hud.label.text = L10n.Hud.ToolChange.eraser
            hud.hide(animated: true, afterDelay: 0.3)
        }
    }
}
