// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit
import Combine

extension SingleCorrectionViewController: ToolbarManagerDelegate {
	func setEmpty(_ empty: Bool) {
        self.correctionPageViewController.setCurrentPageEmpty(empty)
	}

	func editSampleSolution() {
        guard let first = correctionPageViewController.viewControllers?.first as? CorrectionViewController,
              let context = first.context else {
            return
        }
        if let solutionPage = first.page as? CorrectionPageType {
            App.shared.coordinator?.startSampleSolutionEditing(
                atPage: solutionPage.solutionPage.value,
                context: context)
        }
    }

	func save() {
        self.save(correction: correction)
	}

	func toolbarWantsToDismiss() {
        guard App.shared.settings.askBeforeDismissingCorrectionSession else {
            self.dismiss(animated: true, completion: nil)
            return
        }

        // if nothing unsaved -> dismiss right away
        guard correctionPageViewController.correction.hasChanges else {
            self.dismiss(animated: true, completion: nil)
            return
        }

        // create the alert
        let alert = UIAlertController(
            title: L10n.CorrectionController.Alert.DismissChanges.title,
            message: L10n.CorrectionController.Alert.DismissChanges.message(correctionPageViewController.correction.erid),
            preferredStyle: .alert
        )

        // add 'cancel' option -> back to correction
        alert.addAction(
            UIAlertAction(
                title: L10n.CorrectionController.Alert.DismissChanges.cancel,
                style: .default,
                handler: { _ in
                    // actually do nothing...
                })
        )

        // add 'ok' option -> discard
        alert.addAction(
            UIAlertAction(
                title: L10n.CorrectionController.Alert.DismissChanges.ok,
                style: .destructive,
                handler: { [weak self] _ in
                    self?.dismiss(animated: true, completion: nil)
                })
        )

        // present the alert
        self.present(alert, animated: true, completion: nil)
	}

	func expandParentView() {}
	func showPreview() {}

	func presentDrawingOptions(_ viewController: UIViewController) {
		self.present(viewController, animated: true, completion: nil)
	}

    func changeDrawingToolbarPosition(_ newPosition: DrawingToolbarPosition) { }
}
