// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit
import Combine

class SingleCorrectionViewController: UIViewController {
	let correction: CorrectionType
	let cache: PageViewControllerCache<CorrectionViewController>
	var context: CorrectionContext
	var correctionPageViewController: CorrectionPageViewController

	var undoManagerObserver: AnyCancellable?

    var activeSavingHandlers: [SubproblemSavingHandler] = []
    var toastViewController: ToastViewController?

    var pencilInteraction: UIPencilInteraction?
    var isModal = false

	init(correction: CorrectionType) {
		self.correction = correction

		let cache: PageViewControllerCache<CorrectionViewController> = PageViewControllerCache()
		let context = CorrectionContext()
        context.useSharedUndoManager = false
		self.cache = cache
		self.context = context

		correctionPageViewController = CorrectionPageViewController(correction: correction, cache: cache, context: context)

		super.init(nibName: nil, bundle: nil)

		cache.creation = { [weak self] page in
			guard let self = self else { return nil }
			let pageVC = CorrectionViewController(page: page, context: self.context)
			page.prefetch()
			return pageVC
		}

		self.title = correction.erid
	}

	required init?(coder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}

    override func viewDidLoad() {
        super.viewDidLoad()
        self.view.backgroundColor = Asset.correctionBackground.color

		edgesForExtendedLayout = []

		addChild(correctionPageViewController) { (correctionView) in
			self.view.addSubview(correctionView)
			correctionView.pinToSuperview()
		}

        setupPencilInteraction()
    }
}

extension SingleCorrectionViewController: DefaultToastPresentingViewController { }

extension SingleCorrectionViewController: DefaultSavingManager { }
