// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Combine
import Foundation
import TUMexamKit
import UIKit

//extension SolutionPageViewController {
//	func setupToolbarManager() {
//		let manager = ToolbarManager(navigationItem: navigationItem,
//									 context: correctionContext)
//		manager.delegate = self
//		self.toolbarManager = manager
//
//		// observe changes to undo manager
//		undoManagerObserver = correctionContext
//			.$currentUndoManager
//			.receiveOnMain()
//			.assignNoRetain(to: \.currentUndoManager, on: manager)
//
//		manager.setupRightItems()
//	}
//}

extension SolutionPageViewController: ToolbarManagerDelegate {
	func setEmpty(_ empty: Bool) {

	}

	func editSampleSolution() {}

	func save() {
		App.shared.currentExam?.solution.save()
	}

	func toolbarWantsToDismiss() {
		let unsaved = solution?.pages.value.filter { $0.hasChanges } ?? []

		guard unsaved.isEmpty == false else {
			navigationController?.popViewController(animated: true)
			navigationController?.interactivePopGestureRecognizer?.isEnabled = true
			return
		}

		let readableCorrections = unsaved.map({ "\($0.pageNumber)" }).joined(separator: ", ")
		// create the alert
		let alert = UIAlertController(
			title: L10n.CorrectionController.Alert.DismissChanges.title,
			message: L10n.CorrectionController.Alert.DismissChanges.message(readableCorrections),
			preferredStyle: .alert
		)

		// add 'cancel' option -> back to correction
		alert.addAction(
			UIAlertAction(
				title: L10n.CorrectionController.Alert.DismissChanges.cancel,
				style: .default,
				handler: { _ in
					// actually do nothing...
				})
		)

		// add 'ok' option -> discard
		alert.addAction(
			UIAlertAction(
				title: L10n.CorrectionController.Alert.DismissChanges.ok,
				style: .destructive,
				handler: { [weak self] _ in
					self?.correctionContext.currentUndoManager?.popAllUndoActions()
					self?.correctionContext.currentUndoManager?.reset()
					App.shared.currentExam?.solution.resetSubmission()
					self?.navigationController?.popViewController(animated: true)
					self?.navigationController?.interactivePopGestureRecognizer?.isEnabled = true
				})
		)

		// present the alert
		DispatchQueue.main.async { [weak self] in
			self?.present(alert, animated: true, completion: nil)
		}
	}

	func presentDrawingOptions(_ viewController: UIViewController) {
		self.present(viewController, animated: true, completion: nil)
	}

    func changeDrawingToolbarPosition(_ newPosition: DrawingToolbarPosition) { }

	func expandParentView() {}
	func showPreview() {}
}
