// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Combine
import Foundation
import TUMexamKit
import UIKit

final class DiffingContainerViewController: UISplitViewController {

	// MARK: - VCs

	private var diffOverViewVC: DiffNavigationViewController
	private var contentVC: DiffingPageViewController

	private var onExpandSubscriber: AnyCancellable?
	private var indexObserver: AnyCancellable?

	private let pages: [SolutionRejectionModel]

	init(pages: [SolutionRejectionModel]) {
		self.pages = pages
		let dataSource = DiffingDataSource(pagesToDiff: pages)
		diffOverViewVC = .init(dataSource: dataSource)
		contentVC = .init(dataSource: dataSource)
		super.init(nibName: nil, bundle: nil)

		diffOverViewVC.navigationDelegate = contentVC

		viewControllers = [diffOverViewVC, contentVC].map(\.embeddedInNavigationController)
	}

	override func viewDidLoad() {
		super.viewDidLoad()
		onExpandSubscriber = contentVC.shouldExpand.sink { [weak self] in
			guard let self = self else { return }
			let currentDM = self.preferredDisplayMode
			if case UISplitViewController.DisplayMode.automatic = currentDM {
				self.preferredDisplayMode = .secondaryOnly
			} else {
				self.preferredDisplayMode = .automatic
			}
		}

		indexObserver = contentVC.$currentIndex.sink { [weak self] index in
			guard let self = self else { return }
			self.diffOverViewVC.currentIndex = index
		}
	}

	deinit {
		onExpandSubscriber?.cancel()
	}

	required init?(coder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}
}
