// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Combine
import Foundation
import os.log
import TUMexamKit
import UIKit

final class DiffingDataSource {

	@Published
	var localPages: [SolutionPageType] = []

	@Published
	var remotePages: [SolutionPageType] = []

	@Published
	var diffingStateSelection: [DiffSelection] = []

	@Published
	var visitedPages: Set<Int> = .init()

	var rejectionModels: [SolutionRejectionModel]

	init(pagesToDiff: [SolutionRejectionModel]) {
		self.rejectionModels = pagesToDiff
		updatePages()
	}

	func recjectionModelForPage(_ page: SolutionPageType) -> SolutionRejectionModel? {
		return rejectionModels.first { page.pageNumber == $0.page }
	}

	func updatePages() {
		let pageNumbers = rejectionModels.map(\.page)
		guard
			let localPages = App.shared.currentExam?.solution.pages.value
				.filter({ pageNumbers.contains($0.pageNumber) })
				.compactMap({ $0 as? SolutionPageType }),
			let copies = App.shared.currentExam?.solution.getCopyOfPages(pageNumbers) else {
			preconditionFailure("Error creating datasource for solution page merging..")
		}
		self.localPages = localPages
		self.diffingStateSelection = localPages.map { _ in return .merge }
		self.remotePages = copies
		copies.forEach { $0.loadLayerIfPresent() }
	}
}
