// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import TUMexamKit
import UIKit
import UIKitCore

final class SolutionPageDiffCell: UITableViewCell {

	@UsesAutoLayout
	private var stateLabel = UILabel()

	@UsesAutoLayout
	private var pageNumberLabel = UILabel()

	override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
		super.init(style: style, reuseIdentifier: reuseIdentifier)
		setupConstraints()
		backgroundColor = .secondarySystemGroupedBackground
	}

	required init?(coder: NSCoder) {
		fatalError("Storyboards are evil.")
	}

	private func setupConstraints() {
		stateLabel
			.add(to: self)
			.left(to: \.leftAnchor, constant: 16)
			.pinVertically(withInsets: .init(top: 8, left: 0, bottom: 8, right: 0))
			.width(60)
		pageNumberLabel
			.add(to: self)
			.pinVertically(withInsets: .init(top: 8, left: 0, bottom: 8, right: 0))
			.right(to: \.rightAnchor, constant: 8)
			.left(to: \.rightAnchor, of: stateLabel, constant: 8)
	}

	func configureWith(value: SolutionPageType, state: DiffSelection, visited: Bool) {
		pageNumberLabel.text = L10n.Diffing.Splitview.Navigation.Cell.title(value.pageNumber)
		stateLabel.text = state.shortDescription
		stateLabel.textAlignment = .center

		stateLabel.cornerRadius = 4
		stateLabel.backgroundColor = visited ? .systemGreen : .systemRed
	}
}
