// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import TUMexamKit
import UIKit
import UIKitCore

final class PreviewViewController: UIViewController {

	@UsesAutoLayout
	private var mergeImageView = UIImageView()

	@UsesAutoLayout
	private var backgroundImageView = UIImageView()

	let lhs: SolutionPageType
	let rhs: SolutionPageType

	init(lhs: SolutionPageType, rhs: SolutionPageType) {
		self.lhs = lhs
		self.rhs = rhs
		super.init(nibName: nil, bundle: nil)
		setupContraints()
	}

	required init?(coder: NSCoder) {
		fatalError("Storyboards are evil.")
	}

	private func setupContraints() {

		backgroundImageView
			.add(to: view)
			.pinToEdges(withInsets: UIEdgeInsets(top: 8, left: 8, bottom: 8, right: 8))

		mergeImageView
			.add(to: view)
			.pinToEdges(withInsets: UIEdgeInsets(top: 8, left: 8, bottom: 8, right: 8))

		mergeImageView.contentMode = .scaleAspectFill
		backgroundImageView.contentMode = .scaleAspectFill
	}

	override func viewDidLoad() {
		super.viewDidLoad()

        backgroundImageView.image = lhs.scan.image.value

		addOnceAutoresizing(LoadingViewController(style: .page))

		lhs.getDrawingImage(onCompletion: { [weak self] cgImage in
			guard let self = self,
				  let cgImage = cgImage,
				  let rhsImage = self.rhs.layerImage.value else { return }

			let lhsImage = UIImage(cgImage: cgImage)

			let rect = CGRect(x: 0, y: 0, width: lhsImage.size.width, height: lhsImage.size.height)
			let renderer = UIGraphicsImageRenderer(size: lhsImage.size)

			let result = renderer.image { _ in
//				UIColor.white.set()
//				ctx.fill(rect)

				lhsImage.draw(in: rect, blendMode: .normal, alpha: 1)
				rhsImage.draw(in: rect, blendMode: .normal, alpha: 1)
			}

			DispatchQueue.main.async { [weak self] in
				self?.clearChildren()
				self?.mergeImageView.image = result
			}

		})
	}
}
