// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

@available(iOS 14.0, *)
class EditFlagCollectionViewCell: UICollectionViewListCell {
    weak var textField: UITextField!
    weak var colorWell: UIColorWell!

    override init(frame: CGRect) {
        super.init(frame: frame)

        let textField = UITextField().autolayout()
        textField.placeholder = L10n.CorrectionFlags.AddFlag.placeholder
        self.contentView.addSubview(textField)

        textField.addTopConstraint(to: \.topAnchor, constant: 8)
        textField.addBottomConstraint(to: \.bottomAnchor, constant: 8)
        textField.addLeadingConstraint(to: \.leadingAnchor, constant: 16)

        self.textField = textField

        let colorWell = UIColorWell().autolayout()
        colorWell.supportsAlpha = false
        self.contentView.addSubview(colorWell)

        colorWell.addTopConstraint(to: \.topAnchor, constant: 8)
        colorWell.addBottomConstraint(to: \.bottomAnchor, constant: 8)
        colorWell.addLeadingConstraint(to: \.trailingAnchor, of: textField, constant: 8)
        colorWell.addTrailingConstraint(to: \.trailingAnchor, constant: 8)
        colorWell.aspectRatio(heightToWidth: 1.0)

        self.colorWell = colorWell
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func awakeFromNib() {
        super.awakeFromNib()
        // Initialization code
    }
}
