// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class FlagCollectionViewCell: UICollectionViewCell {
    static let reuseIdentifier = "FlagCollectionViewCell"

    let backgroundColorView = UIView().autolayout()
    let label = UILabel().autolayout()
    let imageView = UIImageView().autolayout()
    let disclosureIndicator = UIImageView().autolayout()

    var color: UIColor? {
        get {
            return backgroundColorView.backgroundColor
        }
        set {
            backgroundColorView.backgroundColor = newValue
            self.setNeedsDisplay()
        }
    }

    var showsDetailDisclosureIndicator = true {
        didSet {
            disclosureIndicator.isHidden = !showsDetailDisclosureIndicator
            horizontalDisclosureConstraints.forEach {
                $0.isActive = showsDetailDisclosureIndicator
            }
            self.setNeedsDisplay()
        }
    }

    private var horizontalDisclosureConstraints = [NSLayoutConstraint]()

    override init(frame: CGRect) {
        super.init(frame: frame)

        backgroundColorView.clipsToBounds = true
        backgroundColorView.layer.cornerRadius = 10
        contentView.addSubview(backgroundColorView)
        backgroundColorView.pinToSuperview()

        imageView.contentMode = .scaleAspectFit
        imageView.image = UIImage(systemName: "flag.fill")
        backgroundColorView.addSubview(imageView)

        label.adjustsFontForContentSizeCategory = true
        label.lineBreakMode = .byTruncatingTail
        backgroundColorView.addSubview(label)

        let inset = CGFloat(8)
        imageView.addLeadingConstraint(to: \.leadingAnchor, constant: inset * 2)
        imageView.addTopConstraint(to: \.topAnchor, constant: inset)
        imageView.addBottomConstraint(to: \.bottomAnchor, constant: inset)
        label.addLeadingConstraint(to: \.trailingAnchor, of: imageView, constant: inset)
        label.addTopConstraint(to: \.topAnchor, constant: inset)
        label.addBottomConstraint(to: \.bottomAnchor, constant: inset)
        label.addTrailingConstraint(to: \.trailingAnchor, constant: inset * 2, priority: .defaultLow)
        label.setContentCompressionResistancePriority(.defaultLow, for: .horizontal)
        // This prevents a bug in UICollectionViewCompisitionalLayout which causes a crash when the label gets too large to fit the width of the container
        label.addWidth(200, relation: .equalOrLess, priority: .defaultHigh)

        disclosureIndicator.image = UIImage(systemName: "chevron.right")
        contentView.addSubview(disclosureIndicator)
        let leading = disclosureIndicator.addLeadingConstraint(to: \.trailingAnchor, of: label, constant: inset)
        disclosureIndicator.addTopConstraint(to: \.topAnchor, constant: inset * 1.25)
        let trailing = disclosureIndicator.addTrailingConstraint(to: \.trailingAnchor, constant: inset * 2)
        disclosureIndicator.addBottomConstraint(to: \.bottomAnchor, constant: inset * 1.25)

        horizontalDisclosureConstraints.append(contentsOf: [leading, trailing])
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

}
