// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class TitleSupplementaryView: UICollectionReusableView {
    static let kind = "section-title-kind"

    let label = UILabel().autolayout()
    static let reuseIdentifier = "title-supplementary-reuse-identifier"

    override init(frame: CGRect) {
        super.init(frame: frame)
        configure()
    }
    required init?(coder: NSCoder) {
        fatalError()
    }
}

extension TitleSupplementaryView {
    func configure() {
        addSubview(label)
        label.addLeadingConstraint(to: \.leadingAnchor)
        label.addTrailingConstraint(to: \.trailingAnchor)
        label.addTopConstraint(to: \.topAnchor, relation: .equalOrLess, constant: 16)
        label.addBottomConstraint(to: \.bottomAnchor, relation: .equalOrLess, constant: 16)
        label.textColor = .secondaryLabel
        label.font = UIFont.systemFont(ofSize: 12.0, weight: .medium)
    }
}
