// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

// Taken from https://stackoverflow.com/questions/25738817/removing-duplicate-elements-from-an-array-in-swift
extension Array where Element: Hashable {
    var uniques: Array {
        var buffer = Array()
        var added = Set<Element>()
        for elem in self {
            if !added.contains(elem) {
                buffer.append(elem)
                added.insert(elem)
            }
        }
        return buffer
    }

    /// Keeps "younger" elements, throws out older ones
    var uniquesYoungPrecedence: Array {
        var buffer = Array()
        var added = Set<Element>()
        for elem in self {
            if added.contains(elem) {
                let index = buffer.firstIndex(of: elem)!
                buffer.remove(at: index)
                buffer.append(elem)
            } else {
                buffer.append(elem)
                added.insert(elem)
            }
        }
        return buffer
    }
}

extension Array {
	mutating func popUntil<T: Equatable>(_ matching: T, forKeyPath keyPath: KeyPath<Element, T>) -> Element? {
		var maybeCurrent = popLast()
		while let current = maybeCurrent {
			if current[keyPath: keyPath] == matching {
				return current
			}
			maybeCurrent = popLast()
		}
		return nil
	}
}
