// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import CoreGraphics
import Foundation
import UIKit

// swiftlint:disable identifier_name
func - (l: CGSize, r: CGSize) -> CGSize { CGSize(width: l.width - r.width, height: l.height - r.height) }
func + (l: CGSize, r: CGSize) -> CGSize { CGSize(width: l.width + r.width, height: l.height + r.height) }
func * (l: CGSize, r: CGSize) -> CGSize { CGSize(width: l.width * r.width, height: l.height * r.height) }
func / (l: CGSize, r: CGSize) -> CGSize { CGSize(width: l.width / r.width, height: l.height / r.height) }

func - (l: CGSize, r: CGFloat) -> CGSize { CGSize(width: l.width - r, height: l.height - r) }
func + (l: CGSize, r: CGFloat) -> CGSize { CGSize(width: l.width + r, height: l.height + r) }
func * (l: CGSize, r: CGFloat) -> CGSize { CGSize(width: l.width * r, height: l.height * r) }
func / (l: CGSize, r: CGFloat) -> CGSize { CGSize(width: l.width / r, height: l.height / r) }

func -= ( l: inout CGSize, r: CGSize) { l = l - r }
func += ( l: inout CGSize, r: CGSize) { l = l + r }
func *= ( l: inout CGSize, r: CGSize) { l = l * r }
func /= ( l: inout CGSize, r: CGSize) { l = l / r }

func -= ( l: inout CGSize, r: CGFloat) { l = l - r }
func += ( l: inout CGSize, r: CGFloat) { l = l + r }
func *= ( l: inout CGSize, r: CGFloat) { l = l * r }
func /= ( l: inout CGSize, r: CGFloat) { l = l / r }
// swiftlint:enable identifier_name

extension CGSize {
	init(width: CGFloat) {
		self.init()
		self.width = width
		self.height = 0
	}
	init(height: CGFloat) {
		self.init()
		self.width = 0
		self.height = height
	}

	func with(width: CGFloat) -> CGSize { CGSize(width: width, height: height) }
	func with(height: CGFloat) -> CGSize { CGSize(width: width, height: height) }
	func ceil() -> CGSize { CGSize(width: CoreGraphics.ceil(width), height: CoreGraphics.ceil(height)) }
	func floor() -> CGSize { CGSize(width: CoreGraphics.floor(width), height: CoreGraphics.floor(height)) }
	func round() -> CGSize { CGSize(width: CoreGraphics.round(width), height: CoreGraphics.round(height)) }

	func point() -> CGPoint {
		CGPoint(x: width, y: height)
	}

	func toInts() -> (width: Int, height: Int) {
		(width: Int(width), height: Int(height))
	}
}
