// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit.UIAlertController
import Logging

extension UIAlertController {
    func show(animated flag: Bool, completion: (() -> Void)? = nil) {

        if let title = self.title, let message = self.message {
            // swiftlint:disable:next line_length
            Logger.appDelegate.info("Showing alert controller with title = `\(title)`, message = `\(message)` on top of view hierarchy")
        } else {
            Logger.appDelegate.info("Showing alert controller \(self) on top of view hierarchy")
        }

        if var topController = UIApplication.shared.windows[0].rootViewController {
            while let presentedViewController = topController.presentedViewController {
                topController = presentedViewController
            }
            topController.present(self, animated: true, completion: completion)
        }
    }
}
