// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

extension UIBarButtonItem {
	static func settings(target: Any?, action: Selector?) -> UIBarButtonItem {
        if #available(iOS 14.0, macOS 11.0, macCatalyst 11.0, *) {
            return self.init(image: UIImage(systemName: "gearshape"), style: .plain, target: target, action: action)
        } else {
            return self.init(image: Asset.settings.image, style: .plain, target: target, action: action)
        }
	}

	static func undo(target: Any?, action: Selector?) -> UIBarButtonItem {
		let image = UIImage(systemName: "arrow.uturn.left")!
		return self.init(image: image, style: .plain, target: target, action: action)
	}

	static func redo(target: Any?, action: Selector?) -> UIBarButtonItem {
		let image = UIImage(systemName: "arrow.uturn.right")!
		return self.init(image: image, style: .plain, target: target, action: action)
	}

	static func save(target: Any?, action: Selector?) -> UIBarButtonItem {
		self.init(barButtonSystemItem: .save, target: target, action: action)
	}
}
