// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

extension UINavigationItem {
    func setTitle(title: String, subtitle: String?) {
        let one = UILabel()
        one.text = title
        one.font = UIFont.boldSystemFont(ofSize: 17)
        one.textColor = Asset.barButtonColor.color
        one.sizeToFit()
        let two = UILabel()
        two.text = subtitle
        two.textColor = Asset.barButtonColor.color
        two.font = UIFont.systemFont(ofSize: 12)
        two.textAlignment = .center
        two.sizeToFit()

        let views = subtitle != nil ? [one, two] : [one]

        let stackView = UIStackView(arrangedSubviews: views)
        stackView.distribution = .equalCentering
        stackView.axis = .vertical
        stackView.alignment = .center

        let width = max(one.frame.size.width, two.frame.size.width)
        stackView.frame = CGRect(x: 0, y: 0, width: width, height: 35)

        self.titleView = stackView
    }
}

class NavigationItemTitleSubtitleView: BaseView {
	let titleLabel: UILabel = UILabel().autolayout()
	let subtitleButton: UIButton = UIButton().autolayout()
	let stackView: UIStackView = UIStackView().autolayout()

	override func setup() {
		super.setup()

		titleLabel.font = UIFont.boldSystemFont(ofSize: 17)
		titleLabel.textColor = Asset.barButtonColor.color

        subtitleButton.setTitleColor(Asset.barButtonColor.color, for: .normal)
        subtitleButton.titleLabel?.font = UIFont.systemFont(ofSize: 12)
        subtitleButton.titleLabel?.textAlignment = .center
        subtitleButton.semanticContentAttribute  = .forceRightToLeft
        subtitleButton.setPreferredSymbolConfiguration(.init(pointSize: 12), forImageIn: .normal)
        subtitleButton.imageEdgeInsets = UIEdgeInsets(top: 1, left: 1, bottom: 0, right: 0)
        subtitleButton.isUserInteractionEnabled = false

		stackView.distribution = .equalCentering
		stackView.axis = .vertical
		stackView.alignment = .center
		stackView.addArrangedSubview(titleLabel)
		stackView.addArrangedSubview(subtitleButton)

		self.addSubview(stackView)
		stackView.pinToSuperview()
		stackView.heightAnchor.constraint(equalToConstant: 35).isActive = true
	}

	func set(title: String, subtitle: String?) {
		titleLabel.text = title
		if let subtitle = subtitle {
			subtitleButton.isHidden = false
            subtitleButton.setTitle(subtitle, for: .normal)
            subtitleButton.setImage(UIImage(systemName: "chevron.down"), for: .normal)
		} else {
			subtitleButton.isHidden = true
            subtitleButton.setTitle("", for: .normal)
            subtitleButton.setImage(nil, for: .normal)
		}
	}
}
