// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
#if canImport(UIKit) && !os(watchOS)
import UIKit

public extension UITableView {

	func register<T: UITableViewCell>(cellType: T.Type) {
		register(cellType.self, forCellReuseIdentifier: cellType.reuseId)
	}

	func registerHeaderFooter<T: UITableViewHeaderFooterView>(viewType: T.Type) {
		register(viewType.self, forHeaderFooterViewReuseIdentifier: viewType.reuseId)
	}

	func dequeueReusableCell<T: UITableViewCell>(at indexPath: IndexPath) -> T {
		guard let flipCell = dequeueReusableCell(withIdentifier: T.reuseId, for: indexPath) as? T
			else { fatalError("Couldn't dequeue a UITableViewCell with identifier: \(T.reuseId)") }
		return flipCell
	}

	func dequeueReusableHeaderFooterView<T: UITableViewHeaderFooterView>(withViewType type: T.Type = T.self) -> T {
		guard let headerFooterView = self.dequeueReusableHeaderFooterView(withIdentifier: type.reuseId) as? T
			else { fatalError("Couldn't dequeue a UITableViewHeaderFooterView with identifier: \(type.reuseId)") }
		return headerFooterView
	}

	func reloadVisibleItems(completion: ((Bool) -> Void)? = nil) {
		guard let indexPathsForVisibleRows = indexPathsForVisibleRows,
			!indexPathsForVisibleRows.isEmpty else { return }

		beginUpdates()
		reloadRows(at: indexPathsForVisibleRows, with: .fade)
		endUpdates()
	}
}
#endif
