// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

//taken from https://stackoverflow.com/a/21848247/1065801
extension UIWindow {
	public var visibleViewController: UIViewController? {
		return UIWindow.getVisibleViewControllerFrom(rootViewController)
	}

	public static func getVisibleViewControllerFrom(_ vc: UIViewController?) -> UIViewController? {
		switch vc {
		case let navigationController as UINavigationController:
			return UIWindow.getVisibleViewControllerFrom(navigationController.visibleViewController
                                                            ?? navigationController.topViewController)

		case let tabBarController as UITabBarController:
			return UIWindow.getVisibleViewControllerFrom(tabBarController.selectedViewController)

		case let presentingViewController where vc?.presentedViewController != nil:
			return UIWindow.getVisibleViewControllerFrom(presentingViewController?.presentedViewController)

		default:
			return vc
		}
	}
}
