// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import CoreGraphics

// don't make this extension available outside this file (yet)
fileprivate extension CGImage {
	var bounds: CGRect {
		CGRect(x: 0, y: 0, width: width, height: height)
	}

	func frameUsing(normalized rect: CGRect) -> CGRect {
		let cgWidth = CGFloat(width)
		let cgHeigth = CGFloat(height)
		return CGRect(
			x: rect.origin.x * cgWidth,
			y: rect.origin.y * cgHeigth,
			width: rect.size.width * cgWidth,
			height: rect.size.height * cgHeigth)
	}
}

class CroppedImage {
	let original: CGImage
	private(set) var cropped: CGImage
	private(set) var rect: CGRect

	init(_ original: CGImage, croppedTo rect: CGRect?) {
		self.original = original
		(self.rect, self.cropped) = Self.crop(original, rect: rect)
	}

	convenience init(_ original: CGImage, croppedToNormalized normalized: CGRect) {
		self.init(original, croppedTo: original.frameUsing(normalized: normalized))
	}

	private static func crop(_ original: CGImage, rect: CGRect?) -> (CGRect, CGImage) {
		// no rect -> original image
		guard let rect = rect else { return (original.bounds, original) }

		// crop rect specified, try to crop
		if let cropped = original.cropping(to: rect) {
			return (rect, cropped)
		} else {
			// TODO: maybe use exception or fallback instead?
			fatalError("couldn't crop image to \(rect)")
		}
	}

	func crop(to rect: CGRect) {
		(self.rect, self.cropped) = Self.crop(original, rect: rect)
	}

	func crop(toNormalized normalized: CGRect) {
		crop(to: original.frameUsing(normalized: normalized))
	}
}
