// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import UIKit.UIGestureRecognizerSubclass

class SimpleStrokeGestureRecognizer: UIGestureRecognizer {
	struct StrokeSample {
		let location: CGPoint

		init(location: CGPoint) {
			self.location = location
		}
	}

	var trackedTouch: UITouch?
	var samples: [StrokeSample] = []

	override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent) {
	   if touches.count != 1 {
		  self.state = .failed
	   }

	   // Capture the first touch and store some information about it.
	   if self.trackedTouch == nil {
		  if let firstTouch = touches.first {
			 self.trackedTouch = firstTouch
			 self.addSample(for: firstTouch)
			 state = .began
		  }
	   } else {
		  // Ignore all but the first touch.
		  for touch in touches where touch != self.trackedTouch {
            self.ignore(touch, for: event)
		  }
	   }
	}

	func addSample(for touch: UITouch) {
	   let newSample = StrokeSample(location: touch.location(in: self.view))
	   self.samples.append(newSample)
	}

	override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
	   self.addSample(for: touches.first!)
	   state = .changed
	}

	override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
	   self.addSample(for: touches.first!)
	   state = .ended
	}

	override func touchesCancelled(_ touches: Set<UITouch>, with event: UIEvent?) {
	   self.samples.removeAll()
	   state = .cancelled
	}

	override func reset() {
	   self.samples.removeAll()
	   self.trackedTouch = nil
	}
}
