// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
#if canImport(Sentry)
import Sentry

extension Breadcrumb {
    static func addInteraction(_ message: String) {
        let breadcrumb = Breadcrumb(level: .info, category: "ui.user")
        breadcrumb.message = message
        breadcrumb.type = "user"
        SentrySDK.addBreadcrumb(crumb: breadcrumb)
    }

    static func addNavigation(_ message: String) {
        let breadcrumb = Breadcrumb(level: .info, category: "ui.nav")
        breadcrumb.message = message
        breadcrumb.type = "navigation"
        SentrySDK.addBreadcrumb(crumb: breadcrumb)
    }

    static func addLifecycle(_ message: String) {
        let breadcrumb = Breadcrumb(level: .info, category: "ui.lifecycle")
        breadcrumb.type = "empty"
        breadcrumb.message = message
        SentrySDK.addBreadcrumb(crumb: breadcrumb)
    }
}
#endif

enum BreadcrumbManager {
    static func addInteraction(_ message: String) {
        #if canImport(Sentry)
        Breadcrumb.addInteraction(message)
        #endif
    }

    static func addNavigation(_ message: String) {
        #if canImport(Sentry)
        Breadcrumb.addNavigation(message)
        #endif
    }

    static func addLifecycle(_ message: String) {
        #if canImport(Sentry)
        Breadcrumb.addLifecycle(message)
        #endif
    }
}
