// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Logging
import CocoaLumberjackSwift
import CocoaLumberjackSwiftSupport

let logIdentifier = "de.tum.in.www1.tumexam"

extension Logging.Logger {
    static let appDelegate = Logger(label: logIdentifier + ".appDelegate")
    static let mainUI = Logger(label: logIdentifier + ".mainUI")
    static let settings = Logger(label: logIdentifier + ".settings")
    static let drawing = Logger(label: logIdentifier + ".drawing")
    static let correctionUI = Logger(label: logIdentifier + ".correctionUI")
    static let loadingUI = Logger(label: logIdentifier + ".loadingUI")
    static let undoRedo = Logger(label: logIdentifier + ".undoRedo")
    static let solutionUI = Logger(label: logIdentifier + ".solutionUI")
    static let shibboleth = Logger(label: logIdentifier + ".shibboleth")
    static let measure = Logger(label: logIdentifier + ".measure")
}

class OSLogFormatter: NSObject, DDLogFormatter {
    func format(message logMessage: DDLogMessage) -> String? {
        guard let swiftLogLabel = logMessage.swiftLogInfo?.logger.label else {
            return logMessage.message
        }

        // I want the same appearance as OSLog in the Xcode console
        // although the actual oslog message wont contain the proper subsystem info for console.app
        guard let fakedSubsystemSub = swiftLogLabel.split(separator: ".").last else {
            return logMessage.message
        }
        let capitalizedSub = fakedSubsystemSub.prefix(1).capitalized + fakedSubsystemSub.dropFirst()
        let subsystem = "[\(capitalizedSub)]".padding(toLength: 14, withPad: " ", startingAt: 0)
        return "\(subsystem) \(logMessage.message)"
    }
}
