// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#if canImport(Sentry)
import Foundation
import CocoaLumberjackSwift
import CocoaLumberjackSwiftSupport
import Sentry

class SentryBreadcrumbLogger: DDAbstractLogger {
    override func log(message logMessage: DDLogMessage) {
        var sentryLevel: SentryLevel = .debug
        switch logMessage.level {
        case .debug:
            sentryLevel = .debug
        case .error:
            sentryLevel = .error
        case .info:
            sentryLevel = .info
        case .warning:
            sentryLevel = .warning
        default:
            sentryLevel = .none
        }
        let breadcrumb = Breadcrumb(level: sentryLevel, category: "")
        if let swiftLogLabel = logMessage.swiftLogInfo?.logger.label,
           let fakedSubsystemSub = swiftLogLabel.split(separator: ".").last {
            let capitalizedSub = fakedSubsystemSub.prefix(1).capitalized + fakedSubsystemSub.dropFirst()
            let subsystem = "Log - [\(capitalizedSub)]"
            breadcrumb.category = subsystem
        }
        breadcrumb.message = logMessage.message
        breadcrumb.timestamp = logMessage.timestamp
        SentrySDK.addBreadcrumb(crumb: breadcrumb)
    }
}
#endif
