// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class PlaceholderTextView: UITextView {

    init() {
        super.init(frame: .zero, textContainer: nil)
        self.font = .preferredFont(forTextStyle: .body)
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(textChanged),
                                               name: UITextView.textDidChangeNotification,
                                               object: nil)
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    deinit {
        NotificationCenter.default.removeObserver(self,
                                                  name: UITextView.textDidChangeNotification,
                                                  object: nil)
    }

    static private let placeholderLabelTag = 69 // nice

    var placeholder: String? {
        set {
            guard newValue != nil else {
                return
            }
            if let placeholderLabel = self.viewWithTag(PlaceholderTextView.placeholderLabelTag) as? UILabel {
                placeholderLabel.text = newValue
            } else {
                addPlaceholderLabel(text: newValue)
            }
        }
        get {
            guard let placeholderLabel = self.viewWithTag(PlaceholderTextView.placeholderLabelTag) as? UILabel else {
                return nil
            }
            return placeholderLabel.text
        }
    }

    private func addPlaceholderLabel(text: String? = nil) {
        let placeholderLabel = UILabel().autolayout()
        placeholderLabel.tag = PlaceholderTextView.placeholderLabelTag
        placeholderLabel.font = self.font
        placeholderLabel.textColor = .tertiaryLabel
        placeholderLabel.text = text
        //self.textContainerInset = UIEdgeInsets(top: 8, left: 0, bottom: 8, right: 0)
        self.textContainer.lineFragmentPadding = 0

        self.addSubview(placeholderLabel)
        placeholderLabel.pinToSuperview(UIEdgeInsets(top: 8, left: 0, bottom: 0, right: 0))
    }

    @objc private func textChanged() {
        guard let placeholderLabel = self.viewWithTag(PlaceholderTextView.placeholderLabelTag) as? UILabel else {
            return
        }
        placeholderLabel.isHidden = !self.text.isEmpty
    }

}
