// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct TextView: UIViewRepresentable {
    let placeholder: String?
    @Binding var text: String
    @Binding var textStyle: UIFont.TextStyle

    init(_ placeholder: String? = nil,
         text: Binding<String>,
         textStyle: Binding<UIFont.TextStyle> = .constant(UIFont.TextStyle.body)) {
        self.placeholder = placeholder
        self._text = text
        self._textStyle = textStyle
    }

    func makeCoordinator() -> TextViewCoordinator {
        TextViewCoordinator($text)
    }

    func makeUIView(context: Context) -> PlaceholderTextView {
        let textView = PlaceholderTextView()
        textView.placeholder = placeholder
        textView.font = UIFont.preferredFont(forTextStyle: textStyle)
        textView.autocapitalizationType = .sentences
        textView.isSelectable = true
        textView.isUserInteractionEnabled = true
        textView.backgroundColor = .clear
        textView.delegate = context.coordinator

        return textView
    }

    func updateUIView(_ uiView: PlaceholderTextView, context: Context) {
        uiView.placeholder = placeholder
        uiView.text = text
        uiView.font = UIFont.preferredFont(forTextStyle: textStyle)
    }
}

class TextViewCoordinator: NSObject, UITextViewDelegate {
    @Binding var text: String

    init(_ text: Binding<String>) {
        self._text = text
    }

    func textViewDidChange(_ textView: UITextView) {
        self.text = textView.text
    }
}
