// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit

class CorrectionBoxButtonGroup {

    // MARK: Properties

    var callback: ((CorrectionBoxButton) -> Void)?
    var buttons: [CorrectionBoxButton]
    var selectedButtons: [CorrectionBoxButton] = []

    var selectedButton: CorrectionBoxButton? {
        get {
            return selectedButtons.first
        }
        set {
            selectedButtons.removeAll()
            if let button = newValue {
                selectedButtons.append(button)
            }
        }
    }

    var allowsMultipleChoice: Bool

    // MARK: Initializers

    init(buttons: [CorrectionBoxButton],
         callback: ((CorrectionBoxButton) -> Void)?,
         allowsMultipleChoice: Bool = false) {
        self.buttons = buttons
        self.callback = callback
        self.allowsMultipleChoice = allowsMultipleChoice

        self.buttons.forEach { button in
            if button.isSelected {
                self.selectedButton = button
            }
            button.addTarget(self, action: #selector(touchUpInside(_:)), for: .touchUpInside)
        }
    }

    // MARK: Public functions

    @objc func touchUpInside(_ button: CorrectionBoxButton?) {
        guard let button = button, buttons.contains(button) else {
            selectedButton = nil
            return
        }

        if allowsMultipleChoice {
            if button.isSelected {
                selectedButtons.append(button)
            } else {
                selectedButtons.removeAll(where: { $0 == button })
            }
        } else {
            self.selectedButton = button
            self.buttons.forEach { button in
                if button != selectedButton {
                    button.isSelected = false
                }
            }
        }

        guard let callback = callback else {
            return
        }

        callback(button)
    }

    func updateOverlayMode(_ overlayMode: Bool) {
        self.buttons.forEach({ $0.overlayMode = overlayMode })
    }

}
