// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

@IBDesignable
class SelectableBarButton: UIButton {
	override func layoutSubviews() {
		super.layoutSubviews()

		// this is a bit hacky (could break in future iOS versions)
		//
		// but, in order have a uniform selection background,
		// resize the first UIImageView which is not the actual imageView (which displays the symbol)
		// this view is currently the rounded-rectangle view for the selection indicator
		for view in subviews {
			if let view = view as? UIImageView, view != imageView {
                view.frame = contentRect(forBounds: bounds).inset(by: .init(2))
			}
		}
	}
}

@available(iOS, introduced: 13, obsoleted: 15)
class SelectableBarButtonItem: UIBarButtonItem {
	let button: UIButton
    #if compiler(>=5.5)
    override var isSelected: Bool {
		get {
			return button.isSelected
		}
		set {
			button.isSelected = newValue
		}
	}
    #else
    var isSelected: Bool {
        get {
            return button.isSelected
        }
        set {
            button.isSelected = newValue
        }
    }
    #endif

	var onValueChange: ((Bool) -> Void)?

	convenience init(with image: UIImage, onValueChange: ((Bool) -> Void)?) {
		let button = SelectableBarButton.systemButton(with: image, target: nil, action: nil)
		button.contentEdgeInsets = UIEdgeInsets(top: 2, left: 2, bottom: 2, right: 2)
		self.init(with: button, onValueChange: onValueChange)
	}

	init(with button: UIButton, onValueChange: ((Bool) -> Void)?) {
		self.button = button
		self.onValueChange = onValueChange

		super.init()

		button.addTarget(self, action: #selector(defaultAction(_:)), for: .touchUpInside)
		button.frame = CGRect(origin: .zero, size: CGSize(width: 44, height: 44))
		self.customView = button
	}

	@objc func defaultAction(_ sender: Any) {
        isSelected = !isSelected
		onValueChange?(isSelected)
	}

	required init?(coder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}
}
