// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class ZoomButton: UIButton {

    override init(frame: CGRect) {
        super.init(frame: frame)
        setup()
    }

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setup()
    }

    private func setup() {
        let gestureRecognizer = UILongPressGestureRecognizer(target: self,
                                                             action: #selector(gestureRecognizerFired(recognizer:)))
        gestureRecognizer.minimumPressDuration = 0.05
        gestureRecognizer.cancelsTouchesInView = false
        self.addGestureRecognizer(gestureRecognizer)
    }

    @objc private func gestureRecognizerFired(recognizer: UILongPressGestureRecognizer) {
        switch recognizer.state {
        case .began:
            UIView.animate(withDuration: 0.5,
                           delay: 0,
                           usingSpringWithDamping: 0.8,
                           initialSpringVelocity: 0.2,
                           options: [.beginFromCurrentState],
                           animations: {
                            self.transform = CGAffineTransform(scaleX: 0.95, y: 0.95)
            })
        case .ended, .cancelled:
            UIView.animate(withDuration: 0.5,
                           delay: 0,
                           usingSpringWithDamping: 0.4,
                           initialSpringVelocity: 0.2,
                           options: [.beginFromCurrentState],
                           animations: {
                            self.transform = .identity
            })
        default:
            return
        }
    }

}
