// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit

class StatusIssuesCell: UICollectionViewCell, CellClassReuseIdentifier {

    private var corneredView: ShadowView!
    var imageView: UIImageView!
    var titleLabel: UILabel!
    var label: UILabel!

    override init(frame: CGRect) {
        super.init(frame: frame)
        setup()
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    private func setup() {
        // Cornered View
        let corneredView = ShadowView().autolayout()
        corneredView.clipsToBounds = false
        corneredView.backgroundColor = UIColor.secondarySystemGroupedBackground
        corneredView.layer.cornerRadius = 20.0

        contentView.addSubview(corneredView)
        self.corneredView = corneredView

        // Title Label
        let titleLabel = UILabel().autolayout()
        titleLabel.font = .systemFont(ofSize: 24, weight: .semibold)
        titleLabel.text = "Title"

        contentView.addSubview(titleLabel)
        self.titleLabel = titleLabel

        // Label
        let label = UILabel().autolayout()
        label.numberOfLines = 0
        label.lineBreakMode = .byWordWrapping
        label.text = "Text"

        contentView.addSubview(label)
        self.label = label

        // Image view
        let imageView = UIImageView().autolayout()
        imageView.image = UIImage(systemName: "exclamationmark.triangle")
        imageView.tintColor = .systemYellow

        contentView.addSubview(imageView)
        self.imageView = imageView

        // Constraints
        corneredView.pinToSuperview(UIEdgeInsets(20))

        imageView.addLeadingConstraint(to: \.leadingAnchor, constant: 20+20)
        imageView.centerY(to: \.centerYAnchor)
        imageView.aspectRatio(heightToWidth: 1)
        imageView.width(44)

        titleLabel.addTopConstraint(to: \.topAnchor, constant: 20+10)
        titleLabel.addTrailingConstraint(to: \.trailingAnchor, constant: 20+20)
        titleLabel.addLeadingConstraint(to: \.trailingAnchor, of: imageView, constant: 10)
        titleLabel.setContentHuggingPriority(.defaultHigh, for: .vertical)

        label.addTopConstraint(to: \.bottomAnchor, of: titleLabel)
        label.addLeadingConstraint(to: \.leadingAnchor, of: titleLabel)
        label.addTrailingConstraint(to: \.trailingAnchor, of: titleLabel)
        label.addBottomConstraint(to: \.bottomAnchor, constant: 20+10)
        label.setContentHuggingPriority(.defaultLow, for: .horizontal)
        label.setContentCompressionResistancePriority(.defaultLow, for: .horizontal)
    }

}
