// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

protocol CellClassReuseIdentifier: class {
	static var reuseIdentifier: String { get }
}

extension CellClassReuseIdentifier {
	static var reuseIdentifier: String {
		return "\(type(of: self))"
	}
}

extension UITableView {
	func register<V: CellClassReuseIdentifier>(_ registrable: V.Type) {
		register(registrable, forCellReuseIdentifier: registrable.reuseIdentifier)
	}

	func dequeueReusableCell<V: CellClassReuseIdentifier>(for indexPath: IndexPath) -> V {
		guard let cell = self.dequeueReusableCell(withIdentifier: V.reuseIdentifier, for: indexPath) as? V else {
			fatalError("can't cast cell to it's corresponding type (\(V.reuseIdentifier), \(indexPath)")
		}

		return cell
	}
}
