// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class CenteringScrollView: UIScrollView {
	// WWDC 2010 Session 104
	override func layoutSubviews() {
		super.layoutSubviews()

		guard let content = delegate?.viewForZooming?(in: self) else { return }

		let scrollViewWidth = bounds.size.width
		let scrollViewHeight = bounds.size.height

		var contentFrame = content.frame
		let contentWidth = contentFrame.width
		let contentHeight = contentFrame.height

		if contentWidth < scrollViewWidth {
			contentFrame.origin.x = (scrollViewWidth - contentWidth) * 0.5
		} else {
			contentFrame.origin.x = 0
		}

		if contentHeight < scrollViewHeight {
			contentFrame.origin.y = (scrollViewHeight - contentHeight) * 0.5
		} else {
			contentFrame.origin.y = 0
		}

		content.frame = contentFrame
	}
}
