// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

class ActiveEraserStrokeLayer: ActiveStrokeLayer {

	static let freezeBatchSize: Int = 50
	var frozenIndex: Int = 0
	var frozenImage: CGImage?

	override func setup() {
		super.setup()
		disableAnimations()
	}

	override func draw(in ctx: CGContext) {
		if let frozen = frozenImage {
			// flip image
			ctx.saveGState()
			ctx.translateBy(x: 0, y: bounds.size.height)
			ctx.scaleBy(x: 1.0, y: -1.0)
			ctx.draw(frozen, in: bounds)
			ctx.restoreGState()
		} else {
			ctx.setFillColor(UIColor.black.cgColor)
			ctx.fill(bounds)
		}
		ctx.setBlendMode(.copy)

		let samples = stroke.samples[frozenIndex..<stroke.samples.count]
        ctx.drawStrokeSamples(samples, ink: stroke.ink, size: bounds.size, referenceSize: referenceSize())

	}

	override func strokeChanged() {
		// if more than batch size finished -> update frozen layer
		if (frozenIndex + Self.freezeBatchSize) < stroke.finishedCount {
            let format = UIGraphicsImageRendererFormat.default()
            format.scale = contentsScale
            let renderer = UIGraphicsImageRenderer(size: frame.size, format: format)
            frozenImage = renderer.image { context in
                render(in: context.cgContext)
            }.cgImage
            frozenIndex = stroke.finishedCount
		}
		setNeedsDisplay()
	}
}
