// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

class ActivePenStrokeLayer: ActiveStrokeLayer {
    var frozenStrokeLayer: StrokeLayer
	var frozenIndex: Int {
		return frozenStrokeLayer.stroke.samples.count
	}

	override var contentsScale: CGFloat {
		didSet {
			frozenStrokeLayer.contentsScale = contentsScale
		}
	}

	static let freezeBatchSize: Int = 50
	static let freezeBatchOverlap: Int = 20

    override init(stroke: ActiveStroke, drawingSize: CGSize) {
        frozenStrokeLayer = StrokeLayer(stroke: .empty, drawingSize: drawingSize)
        super.init(stroke: stroke, drawingSize: drawingSize)
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

	override func setup() {
		addSublayer(frozenStrokeLayer)

		let newActions = [
			"onOrderIn": NSNull(),
			"onOrderOut": NSNull(),
			"sublayers": NSNull(),
			"contents": NSNull(),
			"bounds": NSNull()
		]
		self.actions = newActions

        self.opacity = Tool.pen.strokeDisplayOpacity
	}

	override func draw(in ctx: CGContext) {
		let startIndex = max(0, (frozenIndex - Self.freezeBatchOverlap))
		let samples = stroke.samples[startIndex..<stroke.samples.count] + stroke.predictedSamples

		if frozenStrokeLayer.frame != bounds {
			frozenStrokeLayer.frame = self.bounds
		}

		guard samples.count > 0 else {
			return
		}

		ctx.drawStrokeSamples(samples, ink: stroke.ink, size: bounds.size, referenceSize: referenceSize())
	}

	override func strokeChanged() {
		// if more than batch size finished -> update frozen layer
		if (frozenIndex + Self.freezeBatchSize) < stroke.finishedCount {
			frozenStrokeLayer.stroke = stroke.immutableCopy(forPrefix: stroke.finishedCount)
		}
		setNeedsDisplay()
	}
}
