// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

class SnippetLayer: BaseLayer {
    var snippet: SnippetElement {
        didSet {
            setNeedsDisplay()
        }
    }

    var imageLayer: CALayer = CALayer()

    init(snippet: SnippetElement) {
        self.snippet = snippet
        super.init()
    }

    override init(layer: Any) {
        if let other = layer as? SnippetLayer {
            self.snippet = other.snippet
        } else {
            fatalError("Cannot copy instance variables from other layer types")
        }
        super.init(layer: layer)
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func setup() {
        super.setup()

        addSublayer(imageLayer)

        imageLayer.contents = snippet.snippet.image?.cgImage
    }

    override func layoutSublayers() {
        super.layoutSublayers()

        imageLayer.frame = snippet.normalizedRect.mappedTo(size: bounds.size)
    }
}
