// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

class StrokeLayer: BaseLayer {
	var stroke: Stroke {
		didSet {
			setNeedsDisplay()
		}
	}

    let drawingSize: CGSize

    init(stroke: Stroke, opacity: Float = 1.0, drawingSize: CGSize) {
		self.stroke = stroke
        self.drawingSize = drawingSize
		super.init()
        self.opacity = opacity
	}

    override func setup() {
        super.setup()
        contentsScale = UIScreen.main.scale
    }

    override init(layer: Any) {
        if let other = layer as? StrokeLayer {
            self.stroke = other.stroke
            self.drawingSize = other.drawingSize
        } else {
            fatalError("Cannot copy instance variables from other layer type")
        }
        super.init(layer: layer)
    }

	required init?(coder: NSCoder) {
		fatalError("init(coder:) has not been implemented")
	}

	override func draw(in ctx: CGContext) {
        stroke.draw(in: ctx, withSize: bounds.size, referenceSize: referenceSize(), renderMode: .layerComposition)
	}

    func referenceSize() -> CGSize {
        return bounds.size.normalizedIn(size: drawingSize)
    }
}
