// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import MKRingProgressView

class NestedRingProgressView: UIView {

    var ringGradients: [[UIColor]] = [
        [Asset.gradientRed1.color, Asset.gradientRed2.color],
        [Asset.gradientGreen1.color, Asset.gradientGreen2.color],
        [Asset.gradientPurple1.color, Asset.gradientPurple2.color]
    ]

    var progressViews: [RingProgressView] = []
    var progress: [Double] = []

    var ringCount: Int = 0 {
        didSet {
            progressViews.forEach({ $0.removeFromSuperview() })
            progressViews.removeAll()
            (0..<ringCount).forEach({ _ in
                let progressView = RingProgressView()
                progressView.translatesAutoresizingMaskIntoConstraints = false
                progressViews.append(progressView)
            })
            layoutRings()
        }
    }

    var ringWidth: CGFloat = 20

    func layoutRings() {
        for index in 0..<progressViews.count {
            let ringView = progressViews[index]
            let gradient = ringGradients[index]

            ringView.startColor = gradient[0]
            ringView.endColor = gradient[1]
            ringView.gradientImageScale = 0.2
            ringView.ringWidth = ringWidth
            ringView.shadowOpacity = 0.2
            let inset = CGFloat(index) * ringWidth + CGFloat(index) * 2
            let insetRect = self.bounds.insetBy(dx: inset, dy: inset)
            ringView.frame = insetRect
            self.addSubview(ringView)
        }
    }

    override func layoutSubviews() {
        super.layoutSubviews()

        for index in 0..<progressViews.count {
            let ringView = progressViews[index]
            let inset = CGFloat(index) * ringWidth + CGFloat(index) * 2
            let insetRect = self.bounds.insetBy(dx: inset, dy: inset)
            ringView.frame = insetRect
			ringView.ringWidth = ringWidth
		}
	}

	func setUsing(progress array: [Double], animated: Bool = true) {
        progress = array
		ringCount = array.count
		if animated {
			DispatchQueue.main.asyncAfter(deadline: .now() + 0.4) {
				UIView.animate(withDuration: 1,
							   delay: 0.0,
							   usingSpringWithDamping: 0.6,
							   initialSpringVelocity: 0.1,
							   options: [],
							   animations: { [weak self] in
								self?.finallySet()
					}, completion: nil)
			}
		} else {
			finallySet()
		}
	}

	private func finallySet() {
		for (idx, value) in progress.enumerated() {
            guard progressViews.count - 1 >= idx else { return }
			progressViews[idx].progress = value
		}
	}
}
