// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import UIKit
import TUMexamKit

class FlagLegendView: UIView {

    var flagDefinitions: [FlagDefinitionType] = [] {
        didSet {
            updateFlagDefinitions()
        }
    }

    private let blurView = UIVisualEffectView(effect: UIBlurEffect(style: .systemUltraThinMaterial)).autolayout()
    private let scrollView = UIScrollView().autolayout()
    private let stackView = UIStackView().autolayout()

    override init(frame: CGRect) {
        super.init(frame: frame)
        setup()
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    private func setup() {
        self.backgroundColor = .clear
        blurView.cornerRadius = 10
        blurView.clipsToBounds = true
        self.layer.shadowRadius = 5
        self.layer.shadowColor = UIColor.black.cgColor
        self.layer.shadowOpacity = 0.2
        self.layer.shadowOffset = .zero
        // self.layer.shadowPath = CGPath(roundedRect: blurView.frame, cornerWidth: 20, cornerHeight: 20, transform: nil)
        self.clipsToBounds = false

        insertSubview(blurView, at: 0)
        blurView.pinToSuperview()

        addSubview(scrollView)
        scrollView.backgroundColor = .clear
        scrollView.pinToSuperview()

        stackView.axis = .horizontal
        stackView.alignment = .fill
        stackView.distribution = .equalSpacing
        stackView.spacing = 15
        scrollView.addSubview(stackView)
        stackView.pinToSuperview(.init(top: 0, left: 8, bottom: 0, right: 8))
        stackView.addHeight(to: \.heightAnchor, of: scrollView)
    }

    private func updateFlagDefinitions() {
        stackView.arrangedSubviews.forEach {
            $0.removeFromSuperview()
        }

        flagDefinitions.map { def in
            let content = FlagLegendContentView(frame: .zero)
            content.label.text = def.text
            content.imageView.tintColor = UIColor(hex: def.color ?? "#FFFFFF")
            return content
        }.forEach(stackView.addArrangedSubview)
    }

}
