// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct SetupFacultyView: View {
    let setupInfo: SetupInfo

    @State var selection: SetupFaculty?

    var body: some View {
        HStack {
            Spacer().frame(width: 200)
            VStack {
                Text(L10n.Setup.Faculty.title)
                    .font(.largeTitle)
                    .fontWeight(.bold)
                    .padding(.top)
                Image(systemName: "studentdesk")
                    .foregroundColor(Color(.systemIndigo))
                    .font(.system(size: 72))
                    .padding()
                Text(L10n.Setup.Faculty.subtitle)
                    .padding(.bottom, 50)
                HStack {
                    List(SetupFaculty.allCases, id: \.self, rowContent: { faculty in
                        if faculty == .manual {
                            NavigationLink(
                                destination: SetupManualFacultyView(setupInfo: SetupInfo(semester: setupInfo.semester)),
                                label: {
                                Text(faculty.description)
                            })
                        } else {
                            NavigationLink(
                                destination: SetupShortnameView(
                                    setupInfo: SetupInfo(semester: setupInfo.semester, faculty: faculty)
                                ),
                                label: {
                                    Text(faculty.description)
                                })
                        }
                    })
                    .listStyle(.plain)
                }
                Spacer()
            }
            Spacer().frame(width: 200)
        }
    }
}

struct SetupFacultyView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            NavigationView {
                SetupFacultyView(setupInfo: SetupInfo(semester: SetupSemester(year: 2020, term: .winter)))
            }
            .navigationViewStyle(StackNavigationViewStyle())
//            .navigationBarTitleDisplayMode(.inline)
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)

            NavigationView {
                SetupFacultyView(setupInfo: SetupInfo(semester: SetupSemester(year: 2020, term: .winter)))
            }
            .navigationViewStyle(StackNavigationViewStyle())
//            .navigationBarTitleDisplayMode(.inline)
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)
            .environment(\.colorScheme, .dark)
        }
    }
}
