// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct SetupManualFacultyView: View {
    var setupInfo: SetupInfo

    @State var faculty: String = ""

    var body: some View {
        HStack {
            Spacer().frame(width: 200.0)
            VStack {
                Text(L10n.Setup.Faculty.title)
                    .font(.largeTitle)
                    .fontWeight(.bold)
                    .padding(.top)
                Image(systemName: "textformat.abc")
                    .foregroundColor(Color(.systemBlue))
                    .font(.system(size: 72))
                    .padding()
                Text(L10n.Setup.Faculty.subtitle)
                    .multilineTextAlignment(.center)
                    .padding(.bottom, 50)
                Divider()
                HStack {
                    Text(L10n.Setup.Shortname.textfieldLabel)
                    TextField("in", text: $faculty)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)
                }
                Divider()
                Spacer()
            }
            Spacer()
                .frame(width: 200.0)
        }
        .navigationBarItems(trailing: NavigationLink(
            destination: SetupShortnameView(
                setupInfo: SetupInfo(semester: setupInfo.semester, customFaculty: faculty)
            ),
            label: {
                Text(L10n.next)
            }))
    }
}

struct SetupManualFacultyView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            NavigationView {
                SetupManualFacultyView(setupInfo: SetupInfo(semester: SetupSemester(year: 2020,
                                                                                term: .winter),
                                                        faculty: .in))
            }
            .navigationViewStyle(StackNavigationViewStyle())
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)

            NavigationView {
                SetupManualFacultyView(setupInfo: SetupInfo(semester: SetupSemester(year: 2020,
                                                                                term: .winter),
                                                        faculty: .in))
            }
            .navigationViewStyle(StackNavigationViewStyle())
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)
            .environment(\.colorScheme, .dark)
        }
    }
}
