// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct SetupManualView: View {

    @State var fqInstanceName: String = ""

    var body: some View {
        HStack {
            Spacer().frame(width: 200.0)
            VStack {
                Text(L10n.Setup.Manual.title)
                    .font(.largeTitle)
                    .fontWeight(.bold)
                    .padding(.top)
                Image(systemName: "network")
                    .foregroundColor(Color(.systemBlue))
                    .font(.system(size: 72))
                    .padding()
                    .onTapGesture(count: 3) {
                        let urlString = fqInstanceName
                        guard let url = URL(string: urlString) else {
                            UIApplication.shared.sendAction(
                                #selector(UIResponder.resignFirstResponder),
                                to: nil,
                                from: nil,
                                for: nil
                            )
                            return
                        }
                        App.shared.resetKeychain()
                        App.shared.setBackendURL(url)
                        App.shared.coordinator?.showExamSetup()
                    }
                Group {
                    Text(L10n.Setup.Manual.subtitle)
                }
                .multilineTextAlignment(.center)

                Divider()
                    .padding(.top)
                HStack {
                    Text(L10n.Setup.Manual.textfieldLabel)
                    TextField("2021ss-in-grnvs", text: $fqInstanceName)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)
                }
                Divider()
                Spacer()
            }
            Spacer()
                .frame(width: 200.0)
        }
        .navigationBarItems(trailing: Button(L10n.next, action: {
            BreadcrumbManager.addNavigation("SetupManualView - attempting to continue using name \(fqInstanceName)")

            var urlString = fqInstanceName
            if urlString.hasSuffix(".hq.tumexam.de") {
                urlString = urlString.replacingOccurrences(of: ".hq.tumexam.de", with: ".hq-ipad.tumexam.de")
            }

            if !urlString.hasSuffix(".hq-ipad.tumexam.de") {
                urlString += ".hq-ipad.tumexam.de"
            }

            if urlString.hasPrefix("http://") {
                urlString = urlString.replacingOccurrences(of: "http://", with: "https://")
            }

            if !urlString.hasPrefix("https://") {
                urlString = "https://" + urlString
            }

            guard let url = URL(string: urlString) else {
                UIApplication.shared.sendAction(
                    #selector(UIResponder.resignFirstResponder),
                    to: nil,
                    from: nil,
                    for: nil
                )
                return
            }
            App.shared.resetKeychain()
            App.shared.setBackendURL(url)
            App.shared.coordinator?.showExamSetup()
        }))
    }
}

struct SetupManualView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            NavigationView {
                SetupManualView()
            }
            .navigationViewStyle(StackNavigationViewStyle())
//            .navigationBarTitleDisplayMode(.inline)
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)

            NavigationView {
                SetupManualView()
            }
            .navigationViewStyle(StackNavigationViewStyle())
//            .navigationBarTitleDisplayMode(.inline)
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)
            .environment(\.colorScheme, .dark)
        }
    }
}
