// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct SetupSemesterView: View {
    @State var selectedSemester: String?

    var semesters: [SetupSemester] {
        let currentYear = Calendar(identifier: .gregorian).component(.year, from: Date())
        let currentMonth = Calendar(identifier: .gregorian).component(.month, from: Date())
        let currentTerm = currentMonth <= 3 || currentMonth >= 10 ? SetupTerm.winter : SetupTerm.summer

        let currentSemester = SetupSemester(year: currentMonth <= 3 ? currentYear - 1 : currentYear,
                                            term: currentTerm)

        return [currentSemester,
                currentSemester.previousSemester(),
                currentSemester.nextSemester()]
    }

    var body: some View {
        ZStack {
            HStack {
                Spacer()
                    .frame(width: 200.0)
                VStack {
                    Text(L10n.Setup.Semester.title)
                        .font(.largeTitle)
                        .fontWeight(.bold)
                        .padding(.top)
                    Image(systemName: "calendar")
                        .foregroundColor(Color(.systemRed))
                        .font(.system(size: 72))
                        .padding()
                    Text(L10n.Setup.Semester.subtitle)
                        .padding(.bottom, 50)
                    List(semesters, id: \.self) { semester in
                        NavigationLink(
                            destination: SetupFacultyView(setupInfo: SetupInfo(semester: semester)),
                            label: {
                                Text("\(semester.description)")
                            })
                    }
                    .listStyle(.plain)
                }
                Spacer()
                    .frame(width: 200.0)
            }
            VStack {
                Spacer()
                HStack {
                    NavigationLink(
                        destination: SetupManualView(),
                        label: {
                            Image(systemName: "exclamationmark.shield")
                            Text(L10n.Setup.Manual.title)
                        })
                        .foregroundColor(Color(Asset.tuMexamBlue.color))
                        .padding()
                    Spacer()
                }
            }
        }
    }
}

struct SetupSemesterView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            NavigationView {
                SetupSemesterView()
            }
            .navigationViewStyle(StackNavigationViewStyle())
//            .navigationBarTitleDisplayMode(.inline)
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)

            NavigationView {
                SetupSemesterView()
            }
            .navigationViewStyle(StackNavigationViewStyle())
//            .navigationBarTitleDisplayMode(.inline)
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)
            .environment(\.colorScheme, .dark)
        }
    }
}
