// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct SetupShortnameView: View {
    var setupInfo: SetupInfo

    @State var name: String = ""

    var body: some View {
        HStack {
            Spacer().frame(width: 200.0)
            VStack {
                Text(L10n.Setup.Shortname.title)
                    .font(.largeTitle)
                    .fontWeight(.bold)
                    .padding(.top)
                Image(systemName: "textformat.abc")
                    .foregroundColor(Color(.systemBlue))
                    .font(.system(size: 72))
                    .padding()
                Text(L10n.Setup.Shortname.subtitle)
                    .multilineTextAlignment(.center)
                    .padding(.bottom, 50)
                Divider()
                HStack {
                    Text(L10n.Setup.Shortname.textfieldLabel)
                    TextField("grnvs", text: $name)
                        .autocapitalization(.none)
                        .disableAutocorrection(true)
                }
                Divider()
                Spacer()
            }
            Spacer()
                .frame(width: 200.0)
        }
        .navigationBarItems(trailing: Button(L10n.next, action: {
            let setup = SetupInfo(
                semester: setupInfo.semester,
                faculty: setupInfo.faculty,
                customFaculty: setupInfo.customFaculty,
                shortname: name
            )
            BreadcrumbManager.addNavigation("SetupShortnameView - Attempting to continue using info \(setup)")
            guard let url = setup.url else {
                return
            }
            App.shared.resetKeychain()
            App.shared.setBackendURL(url)
            App.shared.coordinator?.showExamSetup()
        }))
    }
}

struct SetupShortnameView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            NavigationView {
                SetupShortnameView(setupInfo: SetupInfo(semester: SetupSemester(year: 2020,
                                                                                term: .winter),
                                                        faculty: .in))
            }
            .navigationViewStyle(StackNavigationViewStyle())
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)

            NavigationView {
                SetupShortnameView(setupInfo: SetupInfo(semester: SetupSemester(year: 2020,
                                                                                term: .winter),
                                                        faculty: .in))
            }
            .navigationViewStyle(StackNavigationViewStyle())
            .modifier(TranslucentNavigationBarModifier())
            .previewLayout(.sizeThatFits)
            .environment(\.colorScheme, .dark)
        }
    }
}
