// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct SetupWelcomeView: View {

    @State var history: [URL] = []

    var body: some View {
        ZStack {
            VStack {
                Image(Asset.roundedAppIcon.name)
                    .resizable()
                    .frame(width: 200.0, height: 200.0)
                Text(L10n.Setup.Welcome.title)
                    .font(.largeTitle)
                    .fontWeight(.semibold)
                Text(L10n.Setup.Welcome.welcome)
                    .foregroundColor(.secondary)
                    .font(.title)
                    .padding()
                NavigationLink(
                    destination: SetupSemesterView(),
                    label: {
                        HStack {
                            Text(L10n.Setup.Welcome.getStarted)
                                .font(.body)
                            Image(systemName: "chevron.right")
                                .font(.body)
                        }.foregroundColor(Color(Asset.barButtonColor.color))
                    })
            }
            if !history.isEmpty {
                VStack {
                    Spacer()
                    HStack {
                        VStack(alignment: .leading) {
                            Text("Connection History")
                                .font(.headline)
                            ForEach(history, id: \.self) { connection in
                                Spacer()
                                    .frame(height: 16)
                                Button {
                                    App.shared.resetKeychain()
                                    App.shared.setBackendURL(connection)
                                    App.shared.coordinator?.showExamSetup()
                                } label: {
                                    HStack {
                                        Text(connection.absoluteString)
                                            .font(.body)
                                        Image(systemName: "chevron.right")
                                            .font(.body)
                                    }.foregroundColor(Color(Asset.barButtonColor.color))
                                }
                                .multilineTextAlignment(.leading)
                            }
                        }
                        .padding()
                        Spacer()
                    }
                }
            }
        }
    }
}

struct SetupWelcomeView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            SetupWelcomeView()
                .previewLayout(.sizeThatFits)
            SetupWelcomeView()
                .previewLayout(.sizeThatFits)
                .environment(\.colorScheme, .dark)
        }
    }
}
