// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct MultiRingView: View {

    let ringGradients: [[Color]] = [
        [Color(Asset.gradientRed1.color), Color(Asset.gradientRed2.color)],
        [Color(Asset.gradientGreen1.color), Color(Asset.gradientGreen2.color)],
        [Color(Asset.gradientPurple1.color), Color(Asset.gradientPurple2.color)]
    ]

    @State var progress: [CGFloat] = []

    var body: some View {
        ZStack {
            ForEach(0..<progress.endIndex, id: \.self) { index in
                RingView(progress: progress[index], colors: ringGradients[index])
                    .padding(CGFloat(22 * index + 2 * index))
            }
        }
    }
}

struct MultiRingView_Previews: PreviewProvider {
    static var previews: some View {
        MultiRingView(progress: [1.0, 0.7])
    }
}
