// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import SwiftUI

struct RingView: View {
    var ringWidth: CGFloat = 20
    var progress: CGFloat
    var colors: [Color]

    var outlineColor: Color {
        colors[0].opacity(0.15)
    }

    @State private var circleRect: CGRect = CGRect()

    var body: some View {
        GeometryReader { geometry in
            ZStack {
                Circle()
                    .stroke(outlineColor, lineWidth: ringWidth)
                    .background(GeometryGetter(rect: $circleRect))
                    .frame(width: geometry.size.smallerSide, height: geometry.size.smallerSide)
                Circle()
                    .trim(from: 0, to: progress)
                    .stroke(
                        AngularGradient(
                            gradient: Gradient(colors: colors),
                            center: .center,
                            startAngle: .degrees(0),
                            endAngle: .degrees(360)
                        ),
                        style: StrokeStyle(lineWidth: ringWidth, lineCap: .round)
                    )
                    .rotationEffect(.degrees(-90))
                    .frame(width: geometry.size.smallerSide, height: geometry.size.smallerSide)
                Circle()
                    .frame(width: ringWidth, height: ringWidth)
                    .foregroundColor(colors[0])
                    .offset(y: -geometry.size.smallerSide / 2)
                Circle()
                    .frame(width: ringWidth, height: ringWidth)
                    .foregroundColor(progress > 0.95 ? colors[1]: colors[1].opacity(0))
                    .offset(y: -geometry.size.smallerSide / 2)
                    .rotationEffect(Angle.degrees(360 * Double(progress)))
                    .shadow(color: progress > 0.96 ? Color.black.opacity(0.1): Color.clear, radius: 3, x: 4, y: 0)
            }
        }
    }
}

struct RingView_Previews: PreviewProvider {
    static var previews: some View {
        RingView(
            progress: 1.0,
            colors: [Color(Asset.gradientRed1.color), Color(Asset.gradientRed2.color)]
        )
    }
}
