// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit

/// small view which draws a horizontal, dashed line
class SubproblemSeparatorView: BaseView {
	override class var layerClass: AnyClass {
		return CAShapeLayer.self
	}
	var shapeLayer: CAShapeLayer {
		// disable force cast (it'll always be a CAShapeLayer, see layerClass)
		//swiftlint:disable force_cast
		return self.layer as! CAShapeLayer
	}

	override func setup() {
		super.setup()

		// style the layer
		shapeLayer.strokeColor = UIColor.separator.cgColor
		shapeLayer.lineWidth = 2.0
		shapeLayer.lineDashPattern = [10.0]
	}

	private var oldBounds: CGRect = .zero

	override func layoutSubviews() {
		super.layoutSubviews()

		// keep track of size and only update if necessary
		if oldBounds != self.bounds {
			let bounds = self.bounds
			let path = CGMutablePath()
			path.move(to: CGPoint(x: bounds.minX, y: bounds.midY))
			path.addLine(to: CGPoint(x: bounds.maxX, y: bounds.midY))
			shapeLayer.path = path

			oldBounds = bounds
		}
	}
}
