# TUMexamKit

This framework contains the logic for loading, editing, and saving corrections.

## Generating the Documentation

You can generate the documentation using [jazzy](https://github.com/realm/jazzy).

Just change the working directory to this one (containing the readme) and enter

```
jazzy
```

Afterwards, the docs can be found it `docs/index.html`.

## Basics

The kit tries to hide as much as possible from any target using it. therefore it relies heavily on public protocols, but internal classes.
Most of the exposed types have a corresponding, internal class (for example `CorrectionType` and `Correction`).
If you're just using the kit, it makes sense to just look at the public protocol types.

## Common Use Cases

The following sections try to provide information on how to achieve common tasks.

### Log In a User

There is no manual log in method in the kit. instead, the networking module tries to initiate a login process as soon as a resource could not be loaded due to
missing authentication.

Therefore, all login-related workflow is made through the `shibbolethAuthentication` module of a `Kit`.
You can find more info in the corresponding class `ShibbolethAuthentication`.

**Important** The delegate has to be set before any resource is loaded.

### Getting the available Exams

The `Kit` provides the available exams using its `examList` property. For more information on the public API, see `ExamListType`.

### Edit and Save a Single Correction

A few steps are necessary to actually get a single correction.

1. You need to load the available exams using the `examList`,  or use `exam(withId:)` to get a specific exam from a `Kit`
2. This exam can be used to create a correction controller for a specific problem/pass or using a more generic filter (see also: *Interactive Correction Mode*)
3. Set the `delegate` and `load()`
4. The delegate is called as soon as corrections were loaded, specifying only the indices, but you can access them using `corrections.value[]`
5. In order to make changes you need to `addLockRequirement()` – which automatically loads the correction once acquired (or even if taken)
6. If locked, you can edit the correction using specific pages (either using `pages.value` or using a convenience method with page number or a problem)
7. Once changed, submit using `submitChanges()`. Take a look at `Submitable` if you are interested in status changes.

### Interactive Correction Mode

If there's more than one person correcting, it is necessary to filter out already-locked corrections and only iterate over available corrections.

This can be done by using the `InteractiveCorrectionPromiseControllerType`, which not only makes one appreciate the autocompletion,
it also filters out anything that cannot be locked. For more information on the internal implementation, look at the corresponding class.

An interactive controller can be created from a `ExamType` object.

While the `CorrectionControllerType` provides the corrections directly, the interactive controller provides  promises (see `CorrectionPromiseType`).
This enables dynamically switching to another correction if the first one could not be locked, without any code in the target.

However, it is necessary to use `getNextCorrection()` and observe the `state` of the promise,
which provides the correction using `.ready(CorrectionType)`.
