// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Combine

@propertyWrapper
public struct InitialPublished<Value>: Publisher {
    public typealias Output = Value
    public typealias Failure = Never

    private var innerValue: Value
    private let subject: PassthroughSubject<Output, Failure>

    public var wrappedValue: Value {
        get { innerValue }
        set {
            innerValue = newValue
            subject.send(newValue)
        }
    }

    public var projectedValue: InitialPublished { self }

    public init(wrappedValue: Value) {
        innerValue = wrappedValue
        subject = PassthroughSubject()
    }

    public func receive<S>(subscriber: S) where S: Subscriber, Value == S.Input, Failure == S.Failure {
        subject.receive(subscriber: subscriber)
//        subject.send(subject.value)
    }
}
