// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

public struct Id<Entity>: Hashable {
	public let raw: String
	public init(_ raw: String) {
		self.raw = raw
	}

	public func hash(into hasher: inout Hasher) {
		hasher.combine(raw)
	}

	public static func == (lhs: Id, rhs: Id) -> Bool {
		lhs.raw == rhs.raw
	}
}

extension Id: Codable {
	public init(from decoder: Decoder) throws {
		let container = try decoder.singleValueContainer()
		let raw = try container.decode(String.self)
		if raw.isEmpty {
			throw DecodingError.dataCorruptedError(
				in: container,
				debugDescription: "Cannot initialize Id from an empty string"
			)
		}
		self.init(raw)
	}

	public func encode(to encoder: Encoder) throws {
		var container = encoder.singleValueContainer()
		try container.encode(raw)
	}
}
