// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import os.log

/// returns the pointer string for a given object
/// - Parameter object: the object
/// - Returns: the pointer as a hex-based string, or nil
public func pointerString<T: AnyObject>(_ object: T?) -> String {
    if let object = object {
        return String(describing: Unmanaged.passUnretained(object).toOpaque())
    } else {
        return "nil"
    }
}

extension OSLog {

    private static var subsystem = Bundle.main.bundleIdentifier!

    static let cache = OSLog(subsystem: subsystem, category: "Cache")
    static let shibboleth = OSLog.disabled // OSLog(subsystem: subsystem, category: "Shibboleth")
    static let network = OSLog(subsystem: subsystem, category: "Network")
    static let lockManager = OSLog(subsystem: subsystem, category: "LockManager")
    static let submission = OSLog(subsystem: subsystem, category: "Submission")
    static let snippets = OSLog(subsystem: subsystem, category: "Snippets")
    static let correction = OSLog(subsystem: subsystem, category: "Correction")
		static let interactive = OSLog(subsystem: subsystem, category: "Interactive")
		static let solution = OSLog(subsystem: subsystem, category: "Solution")
}
