// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import KeychainAccess
import Logging

extension Kit {
    /// loads the default keychain and tries to restore cookies from the keychain. automatically called at startup
    func setupKeychain() {
        let keychain = Keychain.default
        self.keychain = keychain
        //		clearKeychain()
        restoreCookiesFromKeychain()
    }

    /// removes everything in the keychain
    func resetKeychain() {
        guard let keychain = self.keychain else { return }
        try? keychain.removeAll()
    }

    /// removes everything in the keychain
    public class func resetKeychain() {
        try? Keychain.default.removeAll()
    }

    /// tries to restore the cookies from the keychain
    func restoreCookiesFromKeychain() {
        guard let keychain = keychain else { return }

        if let authInfo = keychain.loadAuthInfo() {
            Logger.shibboleth.info("using stored authinfo \(authInfo.debugDescription)")
            networking.setAuthenticationInfo(authInfo)
        }
    }
}
