// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

struct CorrectionDataModel: Codable {
    var examId: Int
    var erid: String

    var passes: Int

    var emptyBoxes: [CorrectionEmptyBoxModel]

    var boxWidth: Int
    var pageWidth: Int
    var pageHeight: Int

    var halfCredits: Bool
    var hasSolution: Bool

    var pages: [Int: CorrectionPageModel]

    var problemDict: [String: CorrectionProblemModel]

    private enum CodingKeys: String, CodingKey {
        case erid, passes, boxWidth, pageWidth, pageHeight, halfCredits, hasSolution, pages, problemDict, emptyBoxes
        case examId = "exam"
    }
}

struct CorrectionPageModel: Codable {
    var landscape: Bool
    var layers: [Int] // Array of layer ids
}

struct CorrectionProblemModel: Codable {
    var boxes: [CorrectionBoxModel]
    var credits: [CorrectionCreditModel]
    var isCredit: Bool // false -> multiple choice
    var page: Int
    var problem: Int
    var subproblem: String
    var xPos: Int
    var yPos: Int

    private enum CodingKeys: String, CodingKey {
        case boxes, credits, page, problem, subproblem
        case isCredit = "is_credit"
        case xPos = "xpos"
        case yPos = "ypos"
    }
}

struct CorrectionCreditModel: Codable {
    var credit: Double?
    var pass: Int
    var corrector: String?
    var date: Date?
    var comment: String?
}

struct CorrectionBoxModel: Codable {
    var boxId: String
    var correctionPass: Int
    var page: Int
    var status: Int
    var value: Double
    var xPos: Int
    var yPos: Int

    var isSelected: Bool {
        return status == 1
    }

    private enum CodingKeys: String, CodingKey {
        case page, status, value
        case boxId = "box_id"
        case correctionPass = "correction_pass"
        case xPos = "xpos"
        case yPos = "ypos"
    }
}

struct CorrectionEmptyBoxModel: Codable {
    var boxId: String
    var page: Int
    var status: Int
    var xPos: Int
    var yPos: Int

    var isSelected: Bool {
        return status == 1
    }

    private enum CodingKeys: String, CodingKey {
        case page, status
        case boxId = "box_id"
        case xPos = "xpos"
        case yPos = "ypos"
    }
}

extension CorrectionDataModel {
    func problems(onPage page: Int) -> [CorrectionProblemModel] {
        return problemDict.values.filter { $0.page == page }
    }

    func page(forProblem problem: Int) -> Int? {
        let problemModels = problemDict.values.filter { (problemModel) -> Bool in
            problemModel.problem == problem
        }
        let firstSubproblem = problemModels.min(by: \.subproblem)
        return firstSubproblem?.page
    }
}
