// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

// MARK: Structs for POSTing to TUMexam

struct CorrectionSubmission: Codable {

    struct CorrectionProblemSubmission: Codable {
        var problem: Int
        var subproblem: String
        var credits: [CorrectionCreditSubmission]
        var comment: String?
        var crosses: [CorrectionBoxSubmission] // Array of checked correction boxes
    }

    struct CorrectionCreditSubmission: Codable {
        var credit: Double
        var correctionPass: Int

        // swiftlint:disable:next nesting
        private enum CodingKeys: String, CodingKey {
            case credit
            case correctionPass = "correction_pass"
        }
    }

    struct CorrectionBoxSubmission: Codable {
        var boxId: String
        private(set) var status: Int

        var boolStatus: Bool {
            get {
                return status == 1
            }
            set {
                status = newValue ? 1 : 0
            }
        }

        // swiftlint:disable:next nesting
        private enum CodingKeys: String, CodingKey {
            case boxId = "box_id"
            case status
        }
    }

    struct CorrectionEmptyBoxSubmission: Codable {
        var boxId: String
        private(set) var status: Int

        var boolStatus: Bool {
            get {
                return status == 1
            }
            set {
                status = newValue ? 1 : 0
            }
        }

        // swiftlint:disable:next nesting
        private enum CodingKeys: String, CodingKey {
            case boxId = "box_id"
            case status
        }
    }

    struct CorrectionImageSubmission: Codable {
        var page: Int
        var layer: Int
        var data: Data
    }

    var lockToken: String
    var problems: [CorrectionProblemSubmission]
    var images: [CorrectionImageSubmission]
    var emptyBoxes: [CorrectionEmptyBoxSubmission]

    private enum CodingKeys: String, CodingKey {
        case lockToken = "lock_token"
        case emptyBoxes = "empty_page"
        case problems, images
    }

}
