// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

struct FlagDefinitionModel: Codable, Hashable {
    var flagTypeId: Int

    var user: String?
    var color: String
    var text: String
    var count: Int?

    private enum CodingKeys: String, CodingKey {
        case flagTypeId = "flag_type_id"
        case color, text, user, count
    }
}

struct FlagDefinitionSubmissionModel: Codable {
    var color: String
    var text: String
}

struct FlagModel: Codable, Hashable {
    var flagId: Int
    var flagTypeId: Int

    var color: String
    var text: String

    var examId: Int
    var erid: String
    var problem: Int?
    var subproblem: String?
    var correctionPass: Int?
    var user: String?

    private enum CodingKeys: String, CodingKey {
        case flagId = "flag_id"
        case flagTypeId = "flag_type_id"
        case correctionPass = "correction_pass"
        case examId = "exam_id"
        case color, text, erid, problem, subproblem, user
    }
}

struct FlagSubmissionModel: Codable {
    // Either this is set...
    var flagTypeId: Int?
    // Or these two are set
    var color: String?
    var text: String?

    var erid: String
    var problem: Int?
    var subproblem: String?
    var correctionPass: Int?

    private enum CodingKeys: String, CodingKey {
        case flagTypeId = "flag_type_id"
        case correctionPass = "correction_pass"
        case color, text, erid, problem, subproblem
    }
}
