// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

struct SolutionDescriptionModel {
	let numberOfPages: Int
	let pagesWithLayers: [Int: SolutionPageDescription]
}

extension SolutionDescriptionModel: Codable {
	private enum FirstLevelKeys: String, CodingKey {
		case layers
		case maxPage = "max_page"
	}

	init(from decoder: Decoder) throws {
		let container = try decoder.container(keyedBy: FirstLevelKeys.self)
		numberOfPages = try container.decode(Int.self, forKey: .maxPage)
		let pages = try container.decode([SolutionPageDescription].self, forKey: .layers)

		pagesWithLayers = pages.reduce(into: [:]) { res, value in
			res[value.pagenum] = value
		}
	}

    func encode(to encoder: Encoder) throws {
        var container = encoder.container(keyedBy: FirstLevelKeys.self)
        try container.encode(numberOfPages, forKey: .maxPage)
        try container.encode(Array(pagesWithLayers.values), forKey: .layers)
    }
}

struct SolutionPageDescription: Codable {
	let layer: Int
	let pagenum: Int
    let date: Date
}
