// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

// TODO refactor public away
public struct SolutionRejectionModel: Equatable {
	public let page: Int
	public let corrector: String?
	public let date: Date

	public static func == (lhs: SolutionRejectionModel, rhs: SolutionRejectionModel) -> Bool {
		return lhs.corrector == rhs.corrector && lhs.page == rhs.page && lhs.date == rhs.date
	}
}

extension SolutionRejectionModel: Decodable {
	private enum CodingKeys: String, CodingKey {
		case page = "pagenum"
		case corrector
		case date
	}

	public init(from decoder: Decoder) throws {
		let container = try decoder.container(keyedBy: CodingKeys.self)
		page = try container.decode(Int.self, forKey: .page)
		corrector = try container.decodeIfPresent(String.self, forKey: .corrector)
		let dateString = try container.decode(String.self, forKey: .date)
		date = DateFormatter.httpHeaderSpec.date(from: dateString) ?? Date()

	}
}
