// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class SolutionLayerImageResource: ImageDownloadResource, IdentifiableResource {
	let identifier: SolutionPageLayerIdentifier
	// don't use cache for layers they might change between loads
	override var useCache: Bool { false }

	internal var onTaskCompleted: ((Date?) -> Void)?

	override var filename: String {
		return "solution-\(identifier.exam)-\(identifier.page)-\(identifier.layer).png"
	}

	override var path: String {
        return "exam/\(identifier.exam)/template/solution/\(identifier.page)/\(identifier.layer)"
	}

	required init(_ identifier: SolutionPageLayerIdentifier, in kit: Kit) {
		self.identifier = identifier
		super.init(in: kit)
	}

	override func shouldFailWithError(forStatusCode statusCode: Int) -> Error? {
		// 500 is returned when layer image does not exist
		if statusCode == 500 {
			return nil
		}
		return super.shouldFailWithError(forStatusCode: statusCode)
	}

	override func taskDidComplete(_ task: URLSessionTask) {
		super.taskDidComplete(task)
		if let response = task.response as? HTTPURLResponse,
           let lastModifiedString = response.value(forHTTPHeaderField: "Last-Modified") {
            let dateFormatter = DateFormatter.httpHeaderSpec
			onTaskCompleted?(dateFormatter.date(from: lastModifiedString))
		}
	}
}
