// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class SubmissionResource: ModelResource<TokenModel> {
    let identifier: CorrectionIdentifier
    let submission: CorrectionSubmission

    init(_ identifier: CorrectionIdentifier, submission: CorrectionSubmission, in kit: Kit) {
        self.identifier = identifier
        self.submission = submission
        super.init(in: kit)
    }

    override func createTask(session: URLSession, networking: Networking) -> URLSessionTask? {
        let path = self.path

        // create post request
        guard var request = networking.protectedRequest(forPath: path) else {
            self.fail(with: NetworkError.unauthorized)
            return nil
        }
        request.httpMethod = "POST"

        // add body
        autoreleasepool {
            let encoder = JSONEncoder()
            encoder.dataEncodingStrategy = .base64
            request.httpBody = try? encoder.encode(submission)
        }

        request.setValue("application/json", forHTTPHeaderField: "Content-Type")

        return session.dataTask(with: request)
    }

    override var path: String {
        "exam/\(identifier.exam)/correction/save/\(identifier.erid)"
    }
}
