// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class ExamCorrectionStatusResource: ModelResource<[CorrectionStatusModel]> {
    let examId: ExamIdentifier
    var perSubproblem: Bool

    override func decoder() -> JSONDecoder {
        let decoder = JSONDecoder()
        decoder.dateDecodingStrategy = .iso8601withConditionalFractionalSeconds
        decoder.keyDecodingStrategy = .convertFromSnakeCase
        return decoder
    }

    init(_ examId: ExamIdentifier, perSubproblem: Bool = false, in kit: Kit) {
        self.examId = examId
        self.perSubproblem = perSubproblem
        super.init(in: kit)
    }

    override func createTask(session: URLSession, networking: Networking) -> URLSessionTask? {
        let path = self.path
        let url = networking.url(forPath: path)
        let queryItems = perSubproblem ? [URLQueryItem(name: "per_subproblem", value: "true")] : []

        guard let finalURL = url.appending(queryItems: queryItems),
              let request = networking.protectedRequest(forURL: finalURL) else {
            self.fail(with: NetworkError.unauthorized)
            return nil
        }

        return session.dataTask(with: request)
    }

    override var path: String {
        "exam/\(examId)/correction/status"
    }
}
