// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class ExamResultsResource: PaginatedModelResource<ExamCorrectionResultModel>, IdentifiableResource {
    struct Filter {
        var erid: String?
        var problem: Int?
        var pass: Int?
    }

    let identifier: ExamIdentifier
    var filter: Filter?

    required init(_ identifier: ExamIdentifier, in kit: Kit) {
        self.identifier = identifier
        super.init(in: kit)
    }

    override func createQueryItems() -> [URLQueryItem] {
        guard let filter = filter else { return super.createQueryItems() }

        var items = super.createQueryItems()
        items.append(contentsOf: filter.queryItems)

        return items
    }

    override var path: String {
        "exam/\(identifier)/result/"
    }
}

extension ExamResultsResource.Filter {
    var queryItems: [URLQueryItem] {
        var items = [URLQueryItem]()
        if let erid = erid {
            items.append(URLQueryItem(name: "filter[erid]", value: erid))
        }
        if let problem = problem {
            items.append(URLQueryItem(name: "filter[problem]", value: "\(problem)"))
        }
        if let pass = pass {
            items.append(URLQueryItem(name: "filter[correction_pass]", value: "\(pass)"))
        }
        return items
    }
}
