// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class FreeLockResource: NetworkDataResource<Bool> {
    struct PostBody: Codable {
        var token: String
    }

    let lockIdentifier: LockIdentifier

    init(_ lockIdentifier: LockIdentifier, in kit: Kit) {
        self.lockIdentifier = lockIdentifier
        super.init(in: kit)
    }

    override func createTask(session: URLSession, networking: Networking) -> URLSessionTask? {
        let path = self.path

        // create post request
        guard var request = networking.protectedRequest(forPath: path) else {
            self.fail(with: NetworkError.unauthorized)
            return nil
        }
        request.httpMethod = "DELETE"

        // add body
        if let kit = kit {
            let body = PostBody.init(token: kit.uuid.uuidString)
            request.httpBody = try? JSONEncoder().encode(body)
            request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        }

        return session.dataTask(with: request)
    }

    override func taskDidComplete(_ task: URLSessionTask) {
        if let task = task.response as? HTTPURLResponse {
            self.value = task.statusCode == 200
            finished()
        } else {
            self.value = false
            finished()
        }
    }

    override var path: String {
        "exam/\(lockIdentifier.exam)/lock/\(lockIdentifier.erid)"
    }
}
