// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class FreeLocksByDeviceResource: NetworkDataResource<Bool> {
    struct PostBody: Codable {
        var token: String
    }

    let identifier: ExamIdentifier

    init(_ identifier: ExamIdentifier, in kit: Kit) {
        self.identifier = identifier
        super.init(in: kit)
    }

    override func createTask(session: URLSession, networking: Networking) -> URLSessionTask? {
        let path = self.path
        var url = networking.urlAgainstBase(forPath: path)

        let queryItems: [URLQueryItem] = createQueryItems()

        if !queryItems.isEmpty {
            url = url.appending(queryItems: queryItems) ?? url
        }

        // add auth
        guard var request = networking.protectedRequest(forURL: url) else {
            self.fail(with: NetworkError.unauthorized)
            return nil
        }
        request.httpMethod = "POST"
        return session.dataTask(with: request)
    }

    func createQueryItems() -> [URLQueryItem] {
        if let kit = kit {
            return [URLQueryItem(name: "filter[token]", value: kit.uuid.uuidString)]
        } else {
            return []
        }
    }

    override func taskDidComplete(_ task: URLSessionTask) {
        if let response = task.response as? HTTPURLResponse {
            self.value = response.statusCode == 200
            //			if let buffer = self.buffer, let string = String(data: buffer, encoding: .utf8) {
            //				print("\(string)")
            //			}

            finished()
        } else {
            self.value = false
            finished()
        }
    }

    override var path: String {
        "exam/\(identifier)/lock/"
    }
}
