// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import os.log

final class SolutionSubmissionResource: ModelResource<[Double]> {
    let id: ExamIdentifier
    let submission: [SolutionSubmissionModel]

    init(_ identifier: ExamIdentifier, submission: [SolutionSubmissionModel], in kit: Kit) {
        self.id = identifier
        self.submission = submission
        super.init(in: kit)
    }

    override func createTask(session: URLSession, networking: Networking) -> URLSessionTask {
        let path = self.path
        let url = networking.url(forPath: path)

        // create post request
        var request = URLRequest(url: url)
        request.httpMethod = "POST"

        // add body
        autoreleasepool {
            let encoder = JSONEncoder()
            encoder.dataEncodingStrategy = .base64
            encoder.dateEncodingStrategy = .iso8601withFractionalSeconds
            request.httpBody = try? encoder.encode(submission)
        }
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")

        return session.dataTask(with: networking.protectedRequest(forRequest: request) ?? request)
    }

    override var path: String {
        "exam/\(id)/template/solution"
    }
}
