// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

/// Identifies the page of an exam
public struct ExamPageIdentifier: Hashable {
    /// the exam
    let exam: ExamIdentifier
    /// the page
    let page: Int
}

extension ExamPageIdentifier: KitIdentifier {
    public var stringIdentifier: String {
        return "E-\(exam)-\(page)"
    }
}

/// Identifies the problem of an exam
public struct ExamProblemIdentifier: Codable, Hashable {
    /// the exam
    let exam: ExamIdentifier
    /// the problem
    let problem: ProblemIdentifier
}

extension ExamProblemIdentifier: KitIdentifier {
    var stringIdentifier: String {
        return "P-\(exam)-\(problem)"
    }
}

/// Identifies the problem/pass of an exam
public struct ExamProblemPassIdentifier: Hashable {
    /// the exam
    let exam: ExamIdentifier
    /// the problem
    let problem: Int
    /// the pass
    let pass: Int
}

extension ExamProblemPassIdentifier: KitIdentifier {
    public var stringIdentifier: String {
        return "P-\(exam)-\(problem)@\(pass)"
    }
}
