// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

/**
	Each exam has a solution with n pages.
	So a solution page is identified by `Sol-(examId)-(page)`.
*/
public struct SolutionPageIdentifier: Hashable {
	let exam: ExamIdentifier
	let page: Int
}

extension SolutionPageIdentifier: KitIdentifier {
	/// Sol-\(exam)-\(page).
	public var stringIdentifier: String {
		return "Sol-\(exam)-\(page)"
	}
}

public struct SolutionPageLayerIdentifier: Hashable {
	let pageIdentifier: SolutionPageIdentifier
	let layer: Int
}

extension SolutionPageLayerIdentifier {
	var exam: ExamIdentifier { return pageIdentifier.exam }
	var page: Int { return pageIdentifier.page }
}

extension SolutionPageLayerIdentifier: KitIdentifier {
	public var stringIdentifier: String {
		return pageIdentifier.stringIdentifier + "-\(layer)"
	}
}
