// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import CoreGraphics

/*

 NOTE:

 this file ports necessary functions for each platform, where they might not be supporte out of the box (mainly macOS, as the kit is iOS-centered)

 */

#if os(iOS)
import UIKit
public typealias Image = UIImage

extension Image {

}

#elseif os(macOS)
import AppKit
public typealias Image = NSImage

// taken and adjusted from https://stackoverflow.com/a/55149310/1065801
extension Image {
    var cgImage: CGImage? {
        self.cgImage(forProposedRect: nil, context: nil, hints: nil)
    }

    convenience init(cgImage: CGImage) {
        self.init(cgImage: cgImage, size: .zero)
    }

    var scale: CGFloat {
        1.0
    }

    func pngData() -> Data? {
        guard let cgImage = self.cgImage else { return nil }

        let imageRep = NSBitmapImageRep(cgImage: cgImage)
        imageRep.size = self.size
        return imageRep.representation(using: .png, properties: [:])
    }
}

#endif
