// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

protocol KitIdentifier: Hashable {
    var stringIdentifier: String { get }
}

protocol KitObjectType: AnyObject {
    associatedtype IdentifierType: KitIdentifier
    var identifier: IdentifierType { get }
}

internal protocol KitChildObjectType: KitObjectType {
    var kit: Kit? { get }

    init(identifier: IdentifierType, kit: Kit)

    static func typedIdentifier(_ identifier: IdentifierType) -> String
}

class KitObject<I: KitIdentifier>: KitChildObjectType {
    public let identifier: I
    weak var kit: Kit?

    required init(identifier: I, kit: Kit) {
        self.identifier = identifier
        self.kit = kit
    }

    class var classIdentifier: ObjectIdentifier {
        return ObjectIdentifier(self)
    }

    class func typedIdentifier(_ identifier: I) -> String {
        return "\(String(describing: Self.self)).\(identifier.stringIdentifier)"
    }
}

extension KitObject: Hashable {
    public static func == (lhs: KitObject<I>, rhs: KitObject<I>) -> Bool {
        return lhs === rhs
    }

    public func hash(into hasher: inout Hasher) {
        return hasher.combine(ObjectIdentifier(self))
    }
}
