// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class CorrectionController {
    let identifier: ExamIdentifier
    unowned var kit: Kit

    weak var delegate: CorrectionControllerDelegate?

    enum InternalState: Equatable {
        case initial // nothing loaded yet
        case pagination(currentPage: Int, maxPage: Int) // in pagination mode
        case retry // pagination failed, changed, done, or empty
        case lookingForUnknownERIDs(currentPage: Int, maxPage: Int) // retrying, but so far knowing all erids
        case done // retry failed
    }

    var filter: ResultFilter {
        didSet {
            if oldValue != filter {
                updateFilter()
            }
        }
    }

    var listMode: Bool = true

    var corrections: [CorrectionType] = []
    var correctionErids: Set<String> = Set()
    var state: Observable<ResourceObjectState> = Observable(.idle)
    var internalState: InternalState = .initial

    var allowModificationsToAllProblems: Bool = false

    lazy var resource: ExamCorrectionsResource = createDelegatedResource()

    var model: PaginationModel<ExamCorrectionModel>?

    init(identifier: ExamIdentifier, filter: ResultFilter = ResultFilter(), kit: Kit) {
        self.identifier = identifier
        self.filter = filter
        self.kit = kit
    }

    public func reset() {
        internalState = .initial
        corrections = []
    }

    func updateFilter() {
        reset()
        self.resource = createDelegatedResource()
    }
}

extension CorrectionController: CorrectionControllerType {
    var estimatedCount: Int {
        guard let model = self.model else { return 0 }
        return model.total
    }
}
