// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

/**
 A small object which provides correction statistics for its owning exam (see `ExamType`).

 It can be loaded using `load()` (see `Loadable`).

 Its state can be observed (see `Stateful`).
 */
public protocol CorrectionStatusType: StatefulLoadable {
    /// the statistics for the problems (observable)
    var problems: Observable<[CorrectionProblemStatusType]> { get }

    /// force reload the statistics
    func forceReload()
}

public protocol CorrectionSubproblemPassStatusType {
    var pass: Int { get }
    var corrected: Int { get }
    var total: Int { get }

    var progress: Double { get }
}

public protocol CorrectionPassStatusType {
    var pass: Int { get }
    var corrected: Int { get }
    var total: Int { get }
    var subCorrected: Int { get }
    var subTotal: Int { get }

    var progress: Double { get }
    var progressRespectingSubproblems: Double { get }
}

public protocol CorrectionProblemStatusType {
    var problem: Int { get }

    var passes: [CorrectionPassStatusType] { get }
    var subproblems: [CorrectionSubproblemStatusType] { get }
}

public protocol CorrectionSubproblemStatusType {
    var subproblem: String { get }

    var passes: [CorrectionSubproblemPassStatusType] { get }
}
