// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import CoreGraphics

extension Drawing.Element {
    func normalized(to rect: CGRect) -> Drawing.Element {
        switch self {
        case let .stroke(stroke):
            return .stroke(stroke.normalized(to: rect))
        case let .snippet(snippet):
        	return .snippet(snippet.normalized(to: rect))
		case let .selection(element):
			return .selection(element.normalized(to: rect))
        }
    }

    func normalized(from rect: CGRect) -> Drawing.Element {
        switch self {
		case let .stroke(stroke):
            return .stroke(stroke.normalized(from: rect))
        case let .snippet(snippet):
			return .snippet(snippet.normalized(from: rect))
		case let .selection(selection):
			return .selection(selection.normalized(from: rect))
        }
    }
}
