// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import CoreGraphics

/// Drawing Element for a selection
public struct SelectionElement {
	/// the selection
	public let selection: Image
	/// the rect
	public let normalizedRect: CGRect

	private let creation = Date()

	/// creates new element with selection and rect
	/// - Parameters:
	///   - selection: the selected portion
	///   - normalizedRect: the rect
	public init(selection: Image, normalizedRect: CGRect) {
		self.selection = selection
		self.normalizedRect = normalizedRect
	}

	func normalized(to rect: CGRect) -> SelectionElement {
		SelectionElement(
			selection: selection,
			normalizedRect: normalizedRect.convertTo(normalizedSubRect: rect))
	}

	func normalized(from rect: CGRect) -> SelectionElement {
		SelectionElement(
			selection: selection,
			normalizedRect: normalizedRect.convertFrom(normalizedSubRect: rect))
	}
}

extension SelectionElement: Equatable {
	public static func == (lhs: SelectionElement, rhs: SelectionElement) -> Bool {
		guard lhs.normalizedRect == rhs.normalizedRect else { return false }
		return lhs.creation == rhs.creation
	}
}
