// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import CoreGraphics

/// Drawing Element for a snippet
public struct SnippetElement {
    /// the snippet
    public let snippet: SnippetType
    /// the rect
    public let normalizedRect: CGRect

    /// creates new element with snippet and rect
    /// - Parameters:
    ///   - snippet: the snippet
    ///   - normalizedRect: the rect
    public init(snippet: SnippetType, normalizedRect: CGRect) {
        self.snippet = snippet
        self.normalizedRect = normalizedRect
    }

    func normalized(to rect: CGRect) -> SnippetElement {
        SnippetElement(snippet: snippet, normalizedRect: normalizedRect.convertTo(normalizedSubRect: rect))
    }

    func normalized(from rect: CGRect) -> SnippetElement {
        SnippetElement(snippet: snippet, normalizedRect: normalizedRect.convertFrom(normalizedSubRect: rect))
    }
}

extension SnippetElement: Equatable {
    public static func == (lhs: SnippetElement, rhs: SnippetElement) -> Bool {
        // check rectangle
        guard lhs.normalizedRect == rhs.normalizedRect else { return false }

        // cast to internal type
        guard let lsnippet = lhs.snippet as? Snippet, let rsnippet = rhs.snippet as? Snippet else { return false }

        // check snippets by identifier
        return lsnippet.identifier == rsnippet.identifier
    }
}
