// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import CoreGraphics

public class Subdrawing: Drawing {
    unowned let parent: Drawing
//    let normalizedRect: CGRect

    init(parent: Drawing, normalizedRect: CGRect) {
        self.parent = parent

        // crop original image and use it as original image (mainly for CALayer stuff)
        let croppedImage: CGImage?
        if let original = parent.originalImage {
            let targetRect = normalizedRect.mappedTo(
                size: CGSize(
                    width: original.width,
                    height: original.height)
            )
            croppedImage = original.cropping(to: targetRect)
        } else {
            croppedImage = nil
        }

        super.init(size: parent.size, originalImage: croppedImage)
        self.normalizedRect = normalizedRect

        updateStrokes()
        parent.delegates.add(self)
    }

    /// see `Drawing`
    /// - Parameter element: the element to add
    public override func append(element: Drawing.Element) {
        let convertedElement = element.normalized(from: normalizedRect)
        parent.append(element: convertedElement)
    }

    public override func removeLast() {
        parent.removeLast()
    }

    private func updateStrokes() {
        self.elements = parent.elements.map { $0.normalized(to: normalizedRect) }
    }
}

extension Subdrawing: DrawingDelegate {
    /// Used internally
    public func drawing(_ drawing: Drawing, didAppendElement element: Element) {
        assert(drawing === parent)

        let local = element.normalized(to: normalizedRect)
        self.elements.append(local)
        delegates.inform { $0.drawing(self, didAppendElement: local) }
    }

    /// Used internally
    public func drawing(_ drawing: Drawing, didRemoveLastElement element: Element) {
        assert(drawing === parent)

        // mirror the behavior in this drawing
        let last = elements.removeLast()
        delegates.inform { $0.drawing(self, didRemoveLastElement: last) }
    }
}
