// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Combine

/// empty identifier to guarantee only one exam list per kit
public struct ExamListIdentifier: KitIdentifier {
    public var stringIdentifier: String {
        return ""
    }
}

class ExamList: KitObject<ExamListIdentifier> {
    lazy var resource: ExamListResource = createDelegatedResource()
    var model: [ExamModel]?
    var state: Observable<ResourceObjectState> = Observable(.idle)

    var exams: Observable<[ExamType]> = Observable([])
}

extension ExamList: ModelResourceKitObjectType {
    func createResource() -> ExamListResource {
        return ExamListResource(in: kit!)
    }

    func update(using model: ModelType) {
        exams.value = model.map({ (model) -> Exam in
            let exam: Exam = kit!.object(for: model.examId)
            exam.update(using: model)
            return exam
        })
    }

    func forceRefresh() {
        state.value = .idle
        model = nil
        resource = createDelegatedResource()
        load()
    }
}

extension ExamList: ShortDescription {
    public var shortDescription: String {
        return "ExamList[state=\(resource.state.shortDescription),exams=\(exams.value.count)]"
    }
}

extension ExamList: ExamListType { }
