// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Combine

public struct FlagDefinitionCreationData {
    var exam: Int
    var color: String
    var text: String
}

public protocol FlagDefinitionType: StatefulLoadable, Submitable, Deletable {
    var flagTypeId: Int { get }
    var user: String? { get }
    var color: String? { get set }
    var text: String? { get set }
    var count: Int? { get }

    func addTo(
        exam: ExamIdentifier,
        erid: String,
        problem: Int?,
        subproblem: String?,
        pass: Int?
    ) -> AnyPublisher<FlagType, Error>

    static func create(data: FlagDefinitionCreationData, in kit: Kit) -> AnyPublisher<FlagDefinitionType, Error>
}
