// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class FlagList: KitObject<FlagListIdentifier> {
    var state: Observable<ResourceObjectState> = Observable(.idle)
    var model: PaginationModel<FlagModel>?
    lazy var resource: FlagsResource = createDelegatedResource()
    var flagList: Observable<[FlagType]> = Observable([])
    var filter: FlagFilter?

    func createResource() -> FlagsResource {
        let res = FlagsResource(exam: identifier.examId, filter: self.filter, in: kit!)
        res.pagingDisabled = true
        return res
    }

    func update(using model: ModelType) {
        self.model = model
        flagList.value = model.results.compactMap({ model -> Flag? in
            let identifier = FlagIdentifier(
                examId: self.identifier.examId,
                flagTypeId: model.flagTypeId,
                flagId: model.flagId
            )
            let flag: Flag? = kit?.object(for: identifier)
            flag?.update(using: model)
            return flag
        })
    }

    func forceReload() {
        state.value = .idle
        model = nil
        resource = createDelegatedResource()
        resource.load()
    }
}

extension FlagList: ModelResourceKitObjectType {
    typealias ModelType = PaginationModel<FlagModel>
    typealias ResourceType = FlagsResource
}
extension FlagList: FlagListType { }
