// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Combine

public struct FlagCreationData {
    var examId: Int
    var flagTypeId: Int?
    var erid: String
    var color: String?
    var text: String?
    var problem: Int?
    var subproblem: String?
    var correctionPass: Int?

    public init(examId: Int,
                flagTypeId: Int? = nil,
                erid: String,
                color: String? = nil,
                text: String? = nil,
                problem: Int? = nil,
                subproblem: String? = nil,
                correctionPass: Int? = nil) {
        self.examId = examId
        self.flagTypeId = flagTypeId
        self.erid = erid
        self.color = color
        self.text = text
        self.problem = problem
        self.subproblem = subproblem
        self.correctionPass = correctionPass
    }
}

public protocol FlagType: StatefulLoadable, Deletable {
    var flagTypeId: Int? { get }
    var flagId: Int? { get }
    var color: String? { get }
    var text: String? { get }
    var erid: String? { get }

    var problem: Int? { get }
    var subproblem: String? { get }
    var correctionPass: Int? { get }
    var user: String? { get }

    static func create(data: FlagCreationData, in kit: Kit) -> AnyPublisher<FlagType, Error>
}
