// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class AccessRights: KitObject<AccessRightsIdentifier> {

    var model: AccessRightsModel?
    lazy var resource: AccessRightsModelResource = createDelegatedResource()
    var state: Observable<ResourceObjectState> = Observable(.idle)

    var uid: String? { model?.uid }
    var name: String? { model?.name }
    var admin: Bool? { model?.admin }
    var correction: Bool? { model?.correction }
    var examManagement: Bool? { model?.examManagement }
    var accessRights: [AccessRightsEntryType] { model?.accessRights ?? [] }

    func createResource() -> AccessRightsModelResource {
        return AccessRightsModelResource(in: kit!)
    }

    func update(using model: AccessRightsModel) {
        self.model = model
    }

    func forceReload() {
        state.value = .idle
        resource = createDelegatedResource()
        resource.load()
    }

    func getAccessRights(_ examId: ExamIdentifier) -> [AccessRightsEntryType] {
        return model?.accessRights.filter { $0.examId == examId } ?? []
    }
}

extension AccessRights: ModelResourceKitObjectType { }
extension AccessRights: AccessRightsType { }
