// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Version

/// placeholder identifier, to ensure only one updateenforcement per kit
public struct UpdateEnforcementIdentifier: KitIdentifier {
    public var stringIdentifier: String {
        return "UPDATE-ENFORCEMENT"
    }
}

class UpdateEnforcement: KitObject<UpdateEnforcementIdentifier> {

    var requiredVersion: Version? {
        return model?.required
    }

    var model: UpdateEnforcementModel?
    lazy var resource: UpdateEnforcementResource = createDelegatedResource()
    var state: Observable<ResourceObjectState> = Observable(.idle)

    func updateRequired(currentVersion version: Version) -> Bool {
        guard let req = requiredVersion else {
            return false
        }
        return version < req
    }
}

extension UpdateEnforcement: ModelResourceKitObjectType {
    func createResource() -> UpdateEnforcementResource {
        return UpdateEnforcementResource(in: kit!)
    }

    func update(using model: UpdateEnforcementModel) {
        self.model = model
    }
}

extension UpdateEnforcement: UpdateEnforcementType { }
