// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import Version

/// placeholder identifier, to ensure only one updateinfo per kit
public struct UpdateInfoIdentifier: KitIdentifier {
    public var stringIdentifier: String {
        return "UPDATE-INFO"
    }
}

class UpdateInfo: KitObject<UpdateInfoIdentifier> {
    lazy var resource: UpdateInfoResource = createDelegatedResource()
    var model: UpdateInfoModel?
    var state: Observable<ResourceObjectState> = Observable(.idle)

    var version: Observable<Version?> = Observable()
}

extension UpdateInfo: ModelResourceKitObjectType {
    func createResource() -> UpdateInfoResource {
        return UpdateInfoResource(in: kit!)
    }

    func update(using model: UpdateInfoModel) {
        self.version.value = model.version
    }
}

extension UpdateInfo: UpdateInfoType { }

extension UpdateInfo: ShortDescription {
    public var shortDescription: String {
        return "UpdateInfo[id=\(identifier),version=\(version.value?.description ?? "(null)")]"
    }
}
