// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

/// the state of a lock
public enum LockState {
    /// we do not have a lock and do not know anything about it
    case unknown
    /// lock already taken
    case taken
    /// lock successfully acquired
    case acquired
    // lock was freed by us. in most cases equivalent to unknown
    case freed
}

public extension LockState {
    /// whether the current state allows editing
    var editable: Bool {
        switch self {
        case .taken, .unknown, .freed:
            return false
        case .acquired:
            return true
        }
    }

    /// whether this state represents the backend's state
    var loaded: Bool {
        switch self {
        case .taken, .acquired:
            return true
        case .unknown, .freed:
            return false
        }
    }
}
