// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import CoreGraphics
import Foundation

/// A single layer on a page
public protocol LayerType: Editable {
	/// the pass of this layer
	var pass: Int { get }

	/// whether this layer can be changed
	var editable: Bool { get }

	/// the loadable image for this layer
	var image: LoadableImageType { get }

	/// the changed image for this layer (can be set, if editable)
	var drawing: Observable<Drawing?> { get }

	/// the underlying scan images size.
	var underlyingScanSize: CGSize? { get set }
}
