// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation

class ProblemPassCorrector: KitObject<CorrectionProblemPassIdentifier> {
    lazy var resource: ExamResultsResource = createDelegatedResource()
    var model: PaginationModel<ExamCorrectionResultModel>?
    var state: Observable<ResourceObjectState> = Observable(.idle)

    var name: Observable<String?> = Observable()
}

extension ProblemPassCorrector: ModelResourceKitObjectType {
    func createResource() -> ExamResultsResource {
        let resource = ExamResultsResource(identifier.correction.exam, in: kit!)
        var filter = ExamResultsResource.Filter()
        filter.erid = identifier.correction.erid
        filter.problem = identifier.problem
        filter.pass = identifier.pass

        resource.filter = filter
        return resource
    }

    func update(using model: ModelType) {
        if let resultModel = model.results.sorted(by: \.date).last {
            name.value = resultModel.corrector
        }
    }
}

extension ProblemPassCorrector: ProblemPassCorrectorType { }

class StaticProblemPassCorrector: ProblemPassCorrectorType {
    var name: Observable<String?> = Observable()
    var state: Observable<ResourceObjectState> = Observable(.loaded)

    init(name: String) {
        self.name.value = name
    }

    func load() {
        // NOP
    }
}
