// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Combine
import Foundation

final class SolutionImage: KitObject<SolutionPageIdentifier> {
    var image: Observable<Image?> = Observable()
    var state: Observable<ResourceObjectState> = Observable(.idle)

	lazy var resource: SolutionImageResource = createDelegatedResource()
}

extension SolutionImage: ResourceKitObjectType {

	func createResource() -> SolutionImageResource {
		return SolutionImageResource(identifier, in: kit!)
	}

	func update(using resource: SolutionImageResource) {
        image.value = resource.createImage()
	}
}

extension SolutionImage: LoadableImageType { }

extension SolutionImage: LockStatePropagation {
    func lockStateChanged(_ lockState: LockState) { }

    func invalidate() {
//        image.value = nil
        resource = createDelegatedResource()
        state.value = .idle
    }
}
