// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
import CoreGraphics

/// Represents a single solution page.
public protocol SolutionPageType: PageType {

	/// An unique exam identifier.
	var erid: String? { get }

	/// the observable layers for this page
	var layers: Observable<[LayerType]> { get }

	var pageNumber: Int { get }
	var pageSize: CGSize { get set }

    var serverLastModified: Date { get }

    var loadableLayerImage: LoadableImageType? { get }
	var layerImage: Observable<Image?> { get }

	var isLayerImageLoaded: Bool { get }
	var imageToMerge: CGImage? { get set }

	func loadLayerIfPresent()
	func getDrawingImage(onCompletion: @escaping ((CGImage?) -> Void))

    func triggerSolutionRefresh()
}

public protocol CorrectionPageType: PageType {
	var solutionPage: Observable<SolutionPageType> { get }
}
