// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import Foundation
@testable import TUMexamKit

extension HTTPCookie {
    convenience init?(name: String, value: String, domain: String, path: String = "/") {
        let cookieProps: [HTTPCookiePropertyKey: Any] = [
            .domain: domain,
            .path: path,
            .name: name,
            .value: value
        ]

        self.init(properties: cookieProps)
    }
}

func setupTestCookies(kit: Kit) {
    // in order to test the networking kit, just copy+paste the variables from safari
    // 1. Open Developer Console
    // 2. Tab "Storage"
    // 3. Select "Cookies" in Side-Bar, if necessary
    // 4. Select all three Cookies and copy (should be X-CSRF-Token, session, proxysession)
    // 5. Paste them here
    //
    // Should look like this:
    // let safariPaste: String = """
    // X-CSRF-Token	SOME_LONG_ID	2019ws-dev-ipad.hq.tumexam.de	/	Session	103 B
    // session	ANOTHER_LONG_ID	2019ws-dev-ipad.hq.tumexam.de	/	Session	138 B		✓
    // proxysession	SOME_REALLY_LONG_ID	2019ws-dev-ipad.hq.tumexam.de	/	Session	280 B	✓	✓
    // """

    //swiftlint:disable line_length
    let safariPaste: String = """
proxysession	Cp3cDDunJ5gyc6caWIxCZIMTW30HtDS5qkzVend8G26D/sC7z7Ju6sc7zNBLbCleqdhvK4ZwufzMBMNZERFJvga1TLVurD2lAOiQC91UBUOKWZQ8V/MCRqOaDVpOTuhB+gVYgc8IXVxgJ4mJ8fvAQaC/diepREywIykqMuLdmqMH7WVIA7thhzAvGrzf4fvuFTc9vhzbp0mto3/QEcvEuBcaFENn24iFKiIwVAlP7IBs7J92Pb3lhJ2VX9IAA5rnVmBXO9UfkG0=	2019ws-dev-ipad.hq.tumexam.de	/	Session	280 B	✓	✓
X-CSRF-Token	IjBhYmZmNmJjZTZlN2Y3ODc1YTE2MDA1OTRlOTYwMGVhZWIzNWRhZDIi.Xtaz-g.QRm_PFhsqq8YXmSfTWE3wfFSaKU	2019ws-dev-ipad.hq.tumexam.de	/	Session	103 B
session	eyJfZnJlc2giOmZhbHNlLCJjc3JmX3Rva2VuIjoiMGFiZmY2YmNlNmU3Zjc4NzVhMTYwMDU5NGU5NjAwZWFlYjM1ZGFkMiJ9.Xtaz_Q.C_7MCxoNklmCK4ssQnlX6lpCi3U	2019ws-dev-ipad.hq.tumexam.de	/	Session	138 B	✓	✓
"""

    var cookies: [HTTPCookie] = []
    for line in safariPaste.components(separatedBy: .newlines) {
        let components = line.components(separatedBy: CharacterSet.whitespaces)
        guard components.count >= 4 else { continue }

        let name = components[0]
        let value = components[1]
        let domain = components[2]
        let path = components[3]

        let cookie = HTTPCookie(name: name, value: value, domain: domain, path: path)
        if let cookie = cookie {
            cookies.append(cookie)
        }
    }

    guard cookies.count == 3 else { fatalError("check your cookies!" ) }
    //	kit.setCookies(cookies)
}

func createTestKit(uuid: UUID? = nil) -> Kit {
    let url = URL(string: "https://2019ws-dev-ipad.hq.tumexam.de")!
    let kit = Kit(backendURL: url, uuid: uuid)

    setupTestCookies(kit: kit)

    return kit
}
