// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

@testable import TUMexamKit

import Foundation
import Quick
import Nimble

//swiftlint:disable force_try
class SnippetTests: QuickSpec {
    override func spec() {
        describe("the snippet") {
            it("can be encoded and decoded") {
                let snippet = Snippet(problem: ExamProblemIdentifier(exam: 1, problem: .subproblem(1, "a")))

                let encoder = JSONEncoder()
                let data = try! encoder.encode(snippet)
                if let string = String(data: data, encoding: .utf8) {
                    print(string)
                }
            }

            context("manager") {
                let kit = createTestKit()
                var manager: SnippetManager!

                beforeEach {
                    manager = SnippetManager(identifier: 1, kit: kit)
                }

                afterEach {
                    manager.reset()
                }

                it("initializes correctly") {
                    manager.load()

                    expect(manager.state.value).toEventually(equal(.loaded))
                    expect(manager.snippets).to(beEmpty())

                    let s1 = Snippet(problem: ExamProblemIdentifier(exam: 1, problem: .subproblem(2, "b")))
                    s1.name = "Test 1"
                    manager.add(s1)

                    manager.reset()
                }
            }
        }
    }
}
