// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#if canImport(UIKit) && !os(watchOS)
import UIKit

public extension Constrainable {
	@discardableResult
	func addWidth(
		_ constant: CGFloat,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint {
		return target.widthAnchor
			.constraint(toConstant: constant, relation: relation)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func width(
		_ constant: CGFloat,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> Self {
		addWidth(constant, relation: relation, priority: priority)
		return self
	}

	@discardableResult
	func addWidth<XAxis>(
		to dimension: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		multiplier: CGFloat = 1,
		constant: CGFloat = 0,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where XAxis: NSLayoutDimension {
		return target.widthAnchor
			.constraint(to: (view ?? parent)[keyPath: dimension], multiplier: multiplier, relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func width<XAxis>(
		to dimension: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		multiplier: CGFloat = 1,
		constant: CGFloat = 0,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> Self where XAxis: NSLayoutDimension {
		addWidth(to: dimension, of: view, multiplier: multiplier, constant: constant, relation: relation, priority: priority)
		return self
	}

	@discardableResult
	func addHeight(
		_ constant: CGFloat,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint {
		return target.heightAnchor
			.constraint(toConstant: constant, relation: relation)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func height(
		_ constant: CGFloat,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> Self {
		addHeight(constant, relation: relation, priority: priority)
		return self
	}

	@discardableResult
	func addHeight<YAxis>(
		to dimension: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		multiplier: CGFloat = 1,
		constant: CGFloat = 0,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where YAxis: NSLayoutDimension {
		return target.heightAnchor
			.constraint(to: (view ?? parent)[keyPath: dimension], multiplier: multiplier, relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func height<YAxis>(
		to dimension: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		multiplier: CGFloat = 1,
		constant: CGFloat = 0,
		relation: ConstraintRelation = .equal,
		priority: UILayoutPriority = .required
	) -> Self where YAxis: NSLayoutDimension {
		addHeight(to: dimension, of: view, multiplier: multiplier, constant: constant, relation: relation, priority: priority)
		return self
	}
}
#endif
