// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#if canImport(UIKit) && !os(watchOS)
import UIKit

public extension Constrainable {
	@discardableResult
	func addLeftConstraint<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where XAxis: NSLayoutXAxisAnchor {
		return target.leftAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func left<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where XAxis: NSLayoutXAxisAnchor {
		addLeftConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}

	@discardableResult
	func addRightConstraint<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where XAxis: NSLayoutXAxisAnchor {
		return target.rightAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(-constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func right<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where XAxis: NSLayoutXAxisAnchor {
		addRightConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}

	@discardableResult
	func addLeadingConstraint<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where XAxis: NSLayoutXAxisAnchor {
		return target.leadingAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func leading<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where XAxis: NSLayoutXAxisAnchor {
		addLeadingConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}

	@discardableResult
	func addTrailingConstraint<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where XAxis: NSLayoutXAxisAnchor {
		return target.trailingAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(-constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func trailing<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where XAxis: NSLayoutXAxisAnchor {
		addTrailingConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}

	@discardableResult
	func addCenterXConstraint<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where XAxis: NSLayoutXAxisAnchor {
		return target.centerXAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func centerX<XAxis>(
		to anchor: KeyPath<UIView, XAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where XAxis: NSLayoutXAxisAnchor {
		addCenterXConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}
}
#endif
