// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#if canImport(UIKit) && !os(watchOS)
import UIKit

public extension Constrainable {
	@discardableResult
	func addTopConstraint<YAxis>(
		to anchor: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where YAxis: NSLayoutYAxisAnchor {
		return target.topAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func top<YAxis>(
		to anchor: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where YAxis: NSLayoutYAxisAnchor {
		addTopConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}

	@discardableResult
	func addBottomConstraint<YAxis>(
		to anchor: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where YAxis: NSLayoutYAxisAnchor {
		return target.bottomAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(-constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func bottom<YAxis>(
		to anchor: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where YAxis: NSLayoutYAxisAnchor {
		addBottomConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}

	@discardableResult
	func addCenterYConstraint<YAxis>(
		to anchor: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> NSLayoutConstraint where YAxis: NSLayoutYAxisAnchor {
		return target.centerYAnchor
			.constraint(to: (view ?? parent)[keyPath: anchor], relation: relation)
			.offset(constant)
			.priority(priority)
			.activate()
	}

	@discardableResult
	func centerY<YAxis>(
		to anchor: KeyPath<UIView, YAxis>,
		of view: UIView? = nil,
		relation: ConstraintRelation = .equal,
		constant: CGFloat = 0,
		priority: UILayoutPriority = .required
	) -> Self where YAxis: NSLayoutYAxisAnchor {
		addCenterYConstraint(to: anchor, of: view, relation: relation, constant: constant, priority: priority)
		return self
	}
}
#endif
