// Copyright 2018-2022 Chair of Network Architectures and Services
//                     Technical University of Munich
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

import UIKit
import TUMexamKit

extension CorrectionCanvasViewController: UIDropInteractionDelegate {
    func setupDrop() {
        let dropInteraction = UIDropInteraction(delegate: self)
        view.addInteraction(dropInteraction)
    }

    func dropInteraction(_ interaction: UIDropInteraction, canHandle session: UIDropSession) -> Bool {
//        return session.canLoadObjects(ofClass: UIImage.self)
        return session.localDragSession != nil
    }

    func dropInteraction(_ interaction: UIDropInteraction, sessionDidUpdate session: UIDropSession) -> UIDropProposal {
        return UIDropProposal(operation: .copy)
    }

    func dropInteraction(_ interaction: UIDropInteraction, performDrop session: UIDropSession) {
        BreadcrumbManager.addInteraction("[CorrectionCanvasVC] user dragndropped snippet")
        let local = session.items.first { (item) -> Bool in
            item.localObject is SnippetType
        }

        guard let snippet = local?.localObject as? SnippetType else { return }

        let location = session.location(in: view)
        let normalized = location.normalized(using: view.bounds.size)
        canvasContext.startSnippetPlacement(snippet: snippet, initialLocation: normalized)
    }
}
